#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from unittest import TestCase

from holidays.calendars.gregorian import MAR
from holidays.constants import GOVERNMENT, UNOFFICIAL
from holidays.countries.northern_mariana_islands import HolidaysMP, MP, MNP
from tests.common import CommonCountryTests


class TestMP(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(HolidaysMP)

    def test_country_aliases(self):
        self.assertAliases(HolidaysMP, MP, MNP)

    def test_mp_only(self):
        """Check for a holiday that is not returned by US unless the
        subdivision is specified."""
        self.assertIn("Commonwealth Covenant Day", self.holidays.get_list(date(2022, MAR, 24)))

    def test_unofficial_holidays(self):
        self.assertHolidays(
            HolidaysMP(categories=UNOFFICIAL, years=2024),
            ("2024-02-14", "Valentine's Day"),
            ("2024-03-17", "Saint Patrick's Day"),
            ("2024-10-31", "Halloween"),
        )

    def test_government_holidays(self):
        self.assertHolidays(
            HolidaysMP(categories=GOVERNMENT, years=2024),
            ("2024-01-01", "New Year's Day"),
            ("2024-01-15", "Birthday of Martin Luther King, Jr."),
            ("2024-02-19", "Washington's Birthday"),
            ("2024-05-27", "Memorial Day"),
            ("2024-06-19", "Juneteenth National Independence Day"),
            ("2024-07-04", "Independence Day"),
            ("2024-09-02", "Labor Day"),
            ("2024-10-14", "Columbus Day"),
            ("2024-11-11", "Veterans Day"),
            ("2024-11-28", "Thanksgiving Day"),
            ("2024-12-25", "Christmas Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

import warnings
from unittest import TestCase

from holidays.countries.france import France, FR, FRA
from tests.common import CommonCountryTests


class TestFrance(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(France)

    def setUp(self):
        super().setUp()
        warnings.simplefilter("ignore", category=DeprecationWarning)

    def test_country_aliases(self):
        self.assertAliases(France, FR, FRA)

    def test_pre_1802(self):
        self.assertNoHoliday("1801-08-15", "1801-11-01", "1801-12-25")

    def test_2017(self):
        self.assertHolidayDates(
            France(years=2017),
            "2017-01-01",
            "2017-04-17",
            "2017-05-01",
            "2017-05-08",
            "2017-05-25",
            "2017-06-05",
            "2017-07-14",
            "2017-08-15",
            "2017-11-01",
            "2017-11-11",
            "2017-12-25",
        )

    def test_2022(self):
        self.assertHolidayDates(
            France(years=2022),
            "2022-01-01",
            "2022-04-18",
            "2022-05-01",
            "2022-05-08",
            "2022-05-26",
            "2022-06-06",
            "2022-07-14",
            "2022-08-15",
            "2022-11-01",
            "2022-11-11",
            "2022-12-25",
        )

    def test_jour_de_lan(self):
        self.assertHoliday("1811-01-01")
        self.assertNoHoliday("1810-01-01")
        self.assertNoHolidayName("Jour de l'an", France(years=1810))

    def test_fete_du_travail(self):
        name_old = "Fête du Travail et de la Concorde sociale"
        name_new = "Fête du Travail"

        self.assertHoliday("1919-05-01")
        self.assertNoHoliday("1918-05-01")
        self.assertNoHolidayName(name_old, France(years=1918))
        self.assertNoHolidayName(name_new, France(years=range(1919, 1948)))
        self.assertNoHolidayName(name_old, France(years=1948))
        self.assertHolidayName(name_old, "1919-05-01", "1947-05-01")
        self.assertHolidayName(name_new, "1948-05-01")

    def test_fete_de_la_victoire(self):
        self.assertHoliday(f"{year}-05-08" for year in range(1953, 1960))
        self.assertHoliday("1982-05-08")
        self.assertNoHoliday("1960-05-08", "1981-05-08")
        self.assertNoHolidayName("Fête de la Victoire", France(years=range(1960, 1982)))

    def test_fete_nationale(self):
        self.assertHoliday("1880-07-14")
        self.assertNoHoliday("1879-07-14")
        self.assertNoHolidayName("Fête nationale", France(years=1879))

    def test_armistice(self):
        self.assertHoliday("1918-11-11")
        self.assertNoHoliday("1917-11-11")
        self.assertNoHolidayName("Armistice", France(years=1917))

    def test_lundi_de_pentecote(self):
        self.assertNoHoliday(
            "2005-05-16",
            "2006-06-05",
            "2007-05-28",
        )
        self.assertHoliday("2004-05-31", "2008-05-12")

    def test_alsace_moselle(self):
        for subdiv in ("GES", "Alsace-Moselle"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-04-14",
                "2017-12-26",
            )

    def test_mayotte(self):
        for subdiv in ("YT", "Mayotte"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-04-27",
            )

    def test_wallis_et_futuna(self):
        for subdiv in ("WF", "Wallis-et-Futuna"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-04-28",
                "2017-07-29",
            )

    def test_martinique(self):
        for subdiv in ("MQ", "Martinique"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-04-14",
                "2017-05-22",
                "2017-07-21",
            )

    def test_guadeloupe(self):
        for subdiv in ("GP", "Guadeloupe"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-04-14",
                "2017-05-27",
                "2017-07-21",
            )

    def test_saint_martin(self):
        for subdiv in ("MF", "Saint-Martin"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2018-05-28",
            )
            self.assertNoHoliday(
                France(subdiv=subdiv),
                "2017-05-28",
            )
            self.assertNoHolidayName("Abolition de l'esclavage", France(subdiv=subdiv, years=2017))

    def test_guyane(self):
        for subdiv in ("GY", "Guyane"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-06-10",
            )

    def test_polynesie_francaise(self):
        for subdiv in ("PF", "Polynésie Française"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-03-05",
                "2017-04-14",
                "2017-06-29",
            )

    def test_nouvelle_caledonie(self):
        for subdiv in ("NC", "Nouvelle-Calédonie"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-09-24",
            )

    def test_saint_barthelemy(self):
        for subdiv in ("BL", "Saint-Barthélémy"):
            self.assertHoliday(France(subdiv=subdiv), "2017-10-09")

    def test_la_reunion(self):
        for subdiv in ("RE", "La Réunion"):
            self.assertHoliday(
                France(subdiv=subdiv),
                "2017-12-20",
            )
            self.assertNoHoliday(
                France(subdiv=subdiv),
                "1980-12-20",
            )
            self.assertNoHolidayName("Abolition de l'esclavage", France(subdiv=subdiv, years=1980))

    def test_subdiv_deprecation(self):
        self.assertDeprecatedSubdivisions("This subdivision is deprecated and will be removed")

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Jour de l'an"),
            ("2022-03-05", "Arrivée de l'Évangile"),
            ("2022-03-24", "Mi-Carême"),
            ("2022-04-15", "Vendredi saint"),
            ("2022-04-18", "Lundi de Pâques"),
            ("2022-04-27", "Abolition de l'esclavage"),
            ("2022-04-28", "Saint Pierre Chanel"),
            ("2022-05-01", "Fête du Travail"),
            ("2022-05-08", "Fête de la Victoire"),
            ("2022-05-22", "Abolition de l'esclavage"),
            ("2022-05-26", "Ascension"),
            ("2022-05-27", "Abolition de l'esclavage"),
            ("2022-05-28", "Abolition de l'esclavage"),
            ("2022-06-06", "Lundi de Pentecôte"),
            ("2022-06-10", "Abolition de l'esclavage"),
            ("2022-06-29", "Fête de l'autonomie"),
            ("2022-07-14", "Fête nationale"),
            ("2022-07-21", "Fête de Victor Schoelcher"),
            ("2022-07-29", "Fête du Territoire"),
            ("2022-08-15", "Assomption"),
            ("2022-09-24", "Fête de la Citoyenneté"),
            ("2022-10-09", "Abolition de l'esclavage"),
            ("2022-11-01", "Toussaint"),
            ("2022-11-11", "Armistice"),
            ("2022-12-20", "Abolition de l'esclavage"),
            ("2022-12-25", "Noël"),
            ("2022-12-26", "Saint Étienne"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-03-05", "Missionary Day"),
            ("2022-03-24", "Mi-Careme"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-27", "Abolition of Slavery"),
            ("2022-04-28", "Feast of Saint Peter Chanel"),
            ("2022-05-01", "Labor Day"),
            ("2022-05-08", "Victory Day"),
            ("2022-05-22", "Abolition of Slavery"),
            ("2022-05-26", "Ascension Day"),
            ("2022-05-27", "Abolition of Slavery"),
            ("2022-05-28", "Abolition of Slavery"),
            ("2022-06-06", "Whit Monday"),
            ("2022-06-10", "Abolition of Slavery"),
            ("2022-06-29", "Internal Autonomy Day"),
            ("2022-07-14", "National Day"),
            ("2022-07-21", "Feast of Victor Schoelcher"),
            ("2022-07-29", "Festival of the Territory"),
            ("2022-08-15", "Assumption Day"),
            ("2022-09-24", "Citizenship Day"),
            ("2022-10-09", "Abolition of Slavery"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-11", "Armistice Day"),
            ("2022-12-20", "Abolition of Slavery"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Saint Stephen's Day"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-03-05", "День місіонерів"),
            ("2022-03-24", "Свято Мі-Карем"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-04-27", "День скасування рабства"),
            ("2022-04-28", "День Святого Пʼєра Шанеля"),
            ("2022-05-01", "День праці"),
            ("2022-05-08", "День Перемоги"),
            ("2022-05-22", "День скасування рабства"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-05-27", "День скасування рабства"),
            ("2022-05-28", "День скасування рабства"),
            ("2022-06-06", "День Святого Духа"),
            ("2022-06-10", "День скасування рабства"),
            ("2022-06-29", "День автономії"),
            ("2022-07-14", "Національне свято"),
            ("2022-07-21", "День Віктора Шольшера"),
            ("2022-07-29", "День Території"),
            ("2022-08-15", "Внебовзяття Пресвятої Діви Марії"),
            ("2022-09-24", "День громадянства"),
            ("2022-10-09", "День скасування рабства"),
            ("2022-11-01", "День усіх святих"),
            ("2022-11-11", "День перемирʼя"),
            ("2022-12-20", "День скасування рабства"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "День Святого Стефана"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import UNOFFICIAL
from holidays.countries.finland import Finland, FI, FIN
from tests.common import CommonCountryTests


class TestFinland(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Finland)
        cls.unofficial_holidays = Finland(categories=UNOFFICIAL, years=range(1853, 2050))

    def test_country_aliases(self):
        self.assertAliases(Finland, FI, FIN)

    def test_fixed_holidays(self):
        for year in range(2010, 2050):
            self.assertHoliday(
                f"{year}-01-01",
                f"{year}-01-06",
                f"{year}-05-01",
                f"{year}-12-06",
                f"{year}-12-24",
                f"{year}-12-25",
                f"{year}-12-26",
            )

    def test_epiphany(self):
        self.assertHolidayName(
            "Loppiainen",
            "1972-01-06",
            "1973-01-06",
            "1974-01-12",
            "1989-01-07",
            "1990-01-06",
            "1991-01-06",
        )
        self.assertNoHoliday(
            "1974-01-06",
            "1975-01-06",
            "1980-01-06",
            "1988-01-06",
            "1989-01-06",
        )

    def test_easter_holidays(self):
        self.assertHoliday(
            # Good Friday
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
            "2023-04-07",
            # Easter Sunday
            "2018-04-01",
            "2019-04-21",
            "2020-04-12",
            "2021-04-04",
            "2022-04-17",
            "2023-04-09",
            # Easter Monday
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
            "2023-04-10",
            # Ascension Day
            "2018-05-10",
            "2019-05-30",
            "2020-05-21",
            "2021-05-13",
            "2022-05-26",
            "2023-05-18",
            # Whit Sunday
            "2018-05-20",
            "2019-06-09",
            "2020-05-31",
            "2021-05-23",
            "2022-06-05",
            "2023-05-28",
        )

    def test_ascension_day(self):
        self.assertHoliday(
            "1971-05-20",
            "1972-05-11",
            "1973-05-26",
            "1974-05-18",
            "1989-04-29",
            "1990-05-19",
            "1991-05-09",
            "1992-05-28",
            "1993-05-20",
        )

        self.assertNoHoliday(
            "1973-05-31",
            "1974-05-23",
            "1980-05-15",
            "1988-05-12",
            "1989-05-04",
            "1990-05-24",
        )

    def test_midsummer_eve(self):
        name = "Juhannusaatto"
        self.assertHolidayName(
            name,
            "1953-06-23",
            "1954-06-23",
            "1955-06-24",
            "1956-06-22",
            "1957-06-21",
        )
        for dt in (
            "1955-06-23",
            "1956-06-23",
            "1957-06-23",
        ):
            self.assertNotIn(name, self.holidays.get(dt, ""))

    def test_midsummer_day(self):
        name = "Juhannuspäivä"
        self.assertHolidayName(
            name,
            "1953-06-24",
            "1954-06-24",
            "1955-06-25",
            "1956-06-23",
            "1957-06-22",
        )
        for dt in (
            "1955-06-24",
            "1956-06-24",
            "1957-06-24",
        ):
            self.assertNotIn(name, self.holidays.get(dt, ""))

    def test_all_saints_day(self):
        self.assertHolidayName(
            "Pyhäinpäivä",
            "1952-11-01",
            "1953-11-01",
            "1954-11-01",
            "1955-11-05",
            "1956-11-03",
            "1957-11-02",
        )
        self.assertNoHoliday(
            "1955-11-01",
            "1956-11-01",
            "1957-11-01",
        )

    def test_independence_day(self):
        name = "Itsenäisyyspäivä"
        self.assertHolidayName(name, (f"{year}-12-06" for year in range(1917, 2050)))
        self.assertNoHoliday(f"{year}-12-06" for year in range(1853, 1917))
        self.assertNoHolidayName(name, range(1853, 1917))

    def _test_unofficial_holiday(self, name, since):
        start_year, month, day = (int(part) for part in since.split("-"))
        self.assertHolidayName(
            name,
            self.unofficial_holidays,
            (f"{year}-{month}-{day}" for year in range(start_year, 2050)),
        )
        self.assertNoHolidayName(name, self.unofficial_holidays, start_year - 1)

    def test_runeberg_day(self):
        self._test_unofficial_holiday("Runebergin päivä", "1854-02-05")

    def test_kalevala_day(self):
        self._test_unofficial_holiday(
            "Kalevalan päivä, suomalaisen kulttuurin päivä", "1860-02-28"
        )

    def test_minna_canth_day(self):
        self._test_unofficial_holiday("Minna Canthin päivä, tasa-arvon päivä", "2007-03-19")

    def test_agricola_day(self):
        self._test_unofficial_holiday("Mikael Agricolan päivä, suomen kielen päivä", "1978-04-09")

    def test_veterans_day(self):
        self._test_unofficial_holiday("Kansallinen veteraanipäivä", "1987-04-27")

    def test_europe_day(self):
        self._test_unofficial_holiday("Eurooppa-päivä", "2019-05-09")

    def test_mothers_day(self):
        self.assertHolidayName(
            "Äitienpäivä",
            self.unofficial_holidays,
            "1918-05-12",
            "1919-05-11",
            "2020-05-10",
            "2024-05-12",
        )
        self.assertNoHoliday(
            self.unofficial_holidays,
            "1917-05-13",
        )

    def test_finnish_identity_day(self):
        self._test_unofficial_holiday("J.V. Snellmanin päivä, suomalaisuuden päivä", "1952-05-12")

    def test_remembrance_day(self):
        self.assertHolidayName(
            "Kaatuneitten muistopäivä",
            self.unofficial_holidays,
            "1977-05-15",
            "1978-05-21",
            "1985-05-19",
            "2024-05-19",
            "2025-05-18",
        )
        self.assertNoHoliday(
            self.unofficial_holidays,
            "1976-05-16",
            "1975-05-18",
        )

    def test_defense_forces_day(self):
        self._test_unofficial_holiday("Puolustusvoimain lippujuhlan päivä", "1942-06-06")

    def test_eino_leino_day(self):
        self._test_unofficial_holiday("Eino Leinon päivä, runon ja suven päivä", "1992-07-06")

    def test_finnish_nature_day(self):
        self.assertHolidayName(
            "Suomen luonnon päivä",
            self.unofficial_holidays,
            "2013-08-31",
            "2014-08-30",
            "2024-08-31",
            "2025-08-30",
            "2026-08-29",
        )
        self.assertNoHoliday(
            self.unofficial_holidays,
            "2012-08-25",
            "2011-08-27",
        )

    def test_miina_sillanpaa_day(self):
        self._test_unofficial_holiday(
            "Miina Sillanpään ja kansalaisvaikuttamisen päivä", "2016-10-01"
        )

    def test_aleksis_kivi_day(self):
        self._test_unofficial_holiday(
            "Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä", "1950-10-10"
        )

    def test_united_nations_day(self):
        self._test_unofficial_holiday("YK:n päivä", "1987-10-24")

    def test_finnish_swedish_heritage(self):
        self._test_unofficial_holiday("Ruotsalaisuuden päivä, Kustaa Aadolfin päivä", "1908-11-06")

    def test_fathers_day(self):
        self.assertHolidayName(
            "Isänpäivä",
            self.unofficial_holidays,
            "1949-11-13",
            "1950-11-12",
            "2020-11-08",
            "2024-11-10",
        )
        self.assertNoHoliday(
            self.unofficial_holidays,
            "1949-11-10",
        )

    def test_day_of_childrens_rights(self):
        self._test_unofficial_holiday("Lapsen oikeuksien päivä", "2020-11-20")

    def test_jean_sibelius_day(self):
        self._test_unofficial_holiday(
            "Jean Sibeliuksen päivä, suomalaisen musiikin päivä", "2007-12-08"
        )

    def test_unofficial_holidays(self):
        self.assertHolidays(
            Finland(categories=UNOFFICIAL, years=2024),
            ("2024-02-05", "Runebergin päivä"),
            ("2024-02-28", "Kalevalan päivä, suomalaisen kulttuurin päivä"),
            ("2024-03-19", "Minna Canthin päivä, tasa-arvon päivä"),
            ("2024-04-09", "Mikael Agricolan päivä, suomen kielen päivä"),
            ("2024-04-27", "Kansallinen veteraanipäivä"),
            ("2024-05-09", "Eurooppa-päivä"),
            ("2024-05-12", "J.V. Snellmanin päivä, suomalaisuuden päivä; Äitienpäivä"),
            ("2024-05-19", "Kaatuneitten muistopäivä"),
            ("2024-06-06", "Puolustusvoimain lippujuhlan päivä"),
            ("2024-07-06", "Eino Leinon päivä, runon ja suven päivä"),
            ("2024-08-31", "Suomen luonnon päivä"),
            ("2024-10-01", "Miina Sillanpään ja kansalaisvaikuttamisen päivä"),
            ("2024-10-10", "Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä"),
            ("2024-10-24", "YK:n päivä"),
            ("2024-11-06", "Ruotsalaisuuden päivä, Kustaa Aadolfin päivä"),
            ("2024-11-10", "Isänpäivä"),
            ("2024-11-20", "Lapsen oikeuksien päivä"),
            ("2024-12-08", "Jean Sibeliuksen päivä, suomalaisen musiikin päivä"),
        )

    def test_2018(self):
        self.assertHolidays(
            ("2018-01-01", "Uudenvuodenpäivä"),
            ("2018-01-06", "Loppiainen"),
            ("2018-03-30", "Pitkäperjantai"),
            ("2018-04-01", "Pääsiäispäivä"),
            ("2018-04-02", "Toinen pääsiäispäivä"),
            ("2018-05-01", "Vappu"),
            ("2018-05-10", "Helatorstai"),
            ("2018-05-20", "Helluntaipäivä"),
            ("2018-06-22", "Juhannusaatto"),
            ("2018-06-23", "Juhannuspäivä"),
            ("2018-11-03", "Pyhäinpäivä"),
            ("2018-12-06", "Itsenäisyyspäivä"),
            ("2018-12-24", "Jouluaatto"),
            ("2018-12-25", "Joulupäivä"),
            ("2018-12-26", "Tapaninpäivä"),
        )

    def test_2022(self):
        self.assertHolidays(
            ("2022-01-01", "Uudenvuodenpäivä"),
            ("2022-01-06", "Loppiainen"),
            ("2022-04-15", "Pitkäperjantai"),
            ("2022-04-17", "Pääsiäispäivä"),
            ("2022-04-18", "Toinen pääsiäispäivä"),
            ("2022-05-01", "Vappu"),
            ("2022-05-26", "Helatorstai"),
            ("2022-06-05", "Helluntaipäivä"),
            ("2022-06-24", "Juhannusaatto"),
            ("2022-06-25", "Juhannuspäivä"),
            ("2022-11-05", "Pyhäinpäivä"),
            ("2022-12-06", "Itsenäisyyspäivä"),
            ("2022-12-24", "Jouluaatto"),
            ("2022-12-25", "Joulupäivä"),
            ("2022-12-26", "Tapaninpäivä"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2022-01-01", "Uudenvuodenpäivä"),
            ("2022-01-06", "Loppiainen"),
            ("2022-02-05", "Runebergin päivä"),
            ("2022-02-28", "Kalevalan päivä, suomalaisen kulttuurin päivä"),
            ("2022-03-19", "Minna Canthin päivä, tasa-arvon päivä"),
            ("2022-04-09", "Mikael Agricolan päivä, suomen kielen päivä"),
            ("2022-04-15", "Pitkäperjantai"),
            ("2022-04-17", "Pääsiäispäivä"),
            ("2022-04-18", "Toinen pääsiäispäivä"),
            ("2022-04-27", "Kansallinen veteraanipäivä"),
            ("2022-05-01", "Vappu"),
            ("2022-05-08", "Äitienpäivä"),
            ("2022-05-09", "Eurooppa-päivä"),
            ("2022-05-12", "J.V. Snellmanin päivä, suomalaisuuden päivä"),
            ("2022-05-15", "Kaatuneitten muistopäivä"),
            ("2022-05-26", "Helatorstai"),
            ("2022-06-05", "Helluntaipäivä"),
            ("2022-06-06", "Puolustusvoimain lippujuhlan päivä"),
            ("2022-06-24", "Juhannusaatto"),
            ("2022-06-25", "Juhannuspäivä"),
            ("2022-07-06", "Eino Leinon päivä, runon ja suven päivä"),
            ("2022-08-27", "Suomen luonnon päivä"),
            ("2022-10-01", "Miina Sillanpään ja kansalaisvaikuttamisen päivä"),
            ("2022-10-10", "Aleksis Kiven päivä, suomalaisen kirjallisuuden päivä"),
            ("2022-10-24", "YK:n päivä"),
            ("2022-11-05", "Pyhäinpäivä"),
            ("2022-11-06", "Ruotsalaisuuden päivä, Kustaa Aadolfin päivä"),
            ("2022-11-13", "Isänpäivä"),
            ("2022-11-20", "Lapsen oikeuksien päivä"),
            ("2022-12-06", "Itsenäisyyspäivä"),
            ("2022-12-08", "Jean Sibeliuksen päivä, suomalaisen musiikin päivä"),
            ("2022-12-24", "Jouluaatto"),
            ("2022-12-25", "Joulupäivä"),
            ("2022-12-26", "Tapaninpäivä"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-02-05", "Runeberg Day"),
            ("2022-02-28", "Kalevala Day, Day of Finnish Culture"),
            ("2022-03-19", "Minna Canth Day, Day of Equality"),
            ("2022-04-09", "Mikael Agricola Day, Day of the Finnish Language"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-04-27", "National War Veterans' Day"),
            ("2022-05-01", "May Day"),
            ("2022-05-08", "Mother's Day"),
            ("2022-05-09", "Europe Day"),
            ("2022-05-12", "J. V. Snellman Day, Day of Finnish Heritage"),
            ("2022-05-15", "Remembrance Day"),
            ("2022-05-26", "Ascension Day"),
            ("2022-06-05", "Whit Sunday"),
            ("2022-06-06", "Flag Day of the Finnish Defense Forces"),
            ("2022-06-24", "Midsummer Eve"),
            ("2022-06-25", "Midsummer Day"),
            ("2022-07-06", "Eino Leino Day, Day of Summer and Poetry"),
            ("2022-08-27", "Finland's Nature Day"),
            ("2022-10-01", "Miina Sillanpää Day, Day of Civic Participation"),
            ("2022-10-10", "Aleksis Kivi Day, Day of Finnish Literature"),
            ("2022-10-24", "United Nations Day"),
            ("2022-11-05", "All Saints' Day"),
            ("2022-11-06", "Finnish Swedish Heritage Day, svenska dagen"),
            ("2022-11-13", "Father's Day"),
            ("2022-11-20", "Day of Children's Rights"),
            ("2022-12-06", "Independence Day"),
            ("2022-12-08", "Jean Sibelius Day, Day of Finnish Music"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Second Day of Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2022-01-01", "Новий рік"),
            ("2022-01-06", "Богоявлення"),
            ("2022-02-05", "День Рунеберга"),
            ("2022-02-28", "День Калевали, День фінської культури"),
            ("2022-03-19", "День Мінни Кант, День рівності"),
            ("2022-04-09", "День Мікаеля Аґріколи, День фінської мови"),
            ("2022-04-15", "Страсна пʼятниця"),
            ("2022-04-17", "Великдень"),
            ("2022-04-18", "Великодній понеділок"),
            ("2022-04-27", "Національний день ветеранів"),
            ("2022-05-01", "Ваппу"),
            ("2022-05-08", "День матері"),
            ("2022-05-09", "День Європи"),
            ("2022-05-12", "День Ю. В. Снелльмана, День фінської спадщини"),
            ("2022-05-15", "День ветеранів Національної війни"),
            ("2022-05-26", "Вознесіння Господнє"),
            ("2022-06-05", "Трійця"),
            ("2022-06-06", "День прапора фінських сил оборони"),
            ("2022-06-24", "Переддень літнього сонцестояння"),
            ("2022-06-25", "День літнього сонцестояння"),
            ("2022-07-06", "День Ейно Лейно, День літа та поезії"),
            ("2022-08-27", "День природи Фінляндії"),
            ("2022-10-01", "День Міїни Сілланпяя, День громадянської активності"),
            ("2022-10-10", "День Алексіса Ківі, День фінської літератури"),
            ("2022-10-24", "День ООН"),
            ("2022-11-05", "День усіх святих"),
            ("2022-11-06", "День фінської шведської спадщини, шведський день"),
            ("2022-11-13", "День батька"),
            ("2022-11-20", "День прав дитини"),
            ("2022-12-06", "День незалежності"),
            ("2022-12-08", "День Жана Сібеліуса, День фінської музики"),
            ("2022-12-24", "Святий вечір"),
            ("2022-12-25", "Різдво Христове"),
            ("2022-12-26", "Другий день Різдва"),
        )

    def test_l10n_sv_fi(self):
        self.assertLocalizedHolidays(
            "sv_FI",
            ("2022-01-01", "Nyårsdagen"),
            ("2022-01-06", "Trettondag"),
            ("2022-02-05", "Runebergsdagen"),
            ("2022-02-28", "Kalevaladagen, den finska kulturens dag"),
            ("2022-03-19", "Minna Canth-dagen, jämställdhetsdagen"),
            ("2022-04-09", "Mikael Agricoladagen, finska språkets dag"),
            ("2022-04-15", "Långfredagen"),
            ("2022-04-17", "Påskdagen"),
            ("2022-04-18", "Annandag påsk"),
            ("2022-04-27", "Nationella veterandagen"),
            ("2022-05-01", "Första maj"),
            ("2022-05-08", "Mors dag"),
            ("2022-05-09", "Europadagen"),
            ("2022-05-12", "Snellmansdagen, finskhetens dag"),
            ("2022-05-15", "De stupades dag"),
            ("2022-05-26", "Kristi himmelsfärdsdag"),
            ("2022-06-05", "Pingst"),
            ("2022-06-06", "Dagen för försvarets fanfest"),
            ("2022-06-24", "Midsommarafton"),
            ("2022-06-25", "Midsommardagen"),
            ("2022-07-06", "Eino Leino-dagen, diktens och sommarens dag"),
            ("2022-08-27", "Den finska naturens dag"),
            ("2022-10-01", "Miina Sillanpää-dagen, medborgarinflytandets dag"),
            ("2022-10-10", "Aleksis Kivi-dagen, den finska litteraturens dag"),
            ("2022-10-24", "FN-dagen"),
            ("2022-11-05", "Alla helgons dag"),
            ("2022-11-06", "Svenska dagen, Gustav Adolfsdagen"),
            ("2022-11-13", "Fars dag"),
            ("2022-11-20", "Barnkonventionens dag"),
            ("2022-12-06", "Självständigshetsdagen"),
            ("2022-12-08", "Sibeliusdagen, den finländska musikens dag"),
            ("2022-12-24", "Julafton"),
            ("2022-12-25", "Juldagen"),
            ("2022-12-26", "Annandag jul"),
        )



#!/usr/bin/env python3

#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from datetime import timedelta as td
from functools import lru_cache
from pathlib import Path


class _Lunisolar:
    def __init__(self) -> None:
        """
        This class generates Gregorian dates for Chinese lunisolar calendar
        based holidays.

        See `Wikipedia
        <https://en.wikipedia.org/wiki/Chinese_New_Year#Dates_in_Chinese_\
        lunisolar_calendar>`__

        Sources:
        http://putidea.blogspot.com/2010/10/python.html
        https://github.com/BohanZhang/python-MagicLamp/blob/master/bin/utils/lunar_calendar.py
        """

        # A binary representation starting from year 1901 of the number of
        # days per year, and the number of days from the 1st to the 13th to
        # store the monthly (including the month of the month). 1 means that
        # the month is 30 days. 0 means the month is 29 days.
        # The 12th to 15th digits indicate the month of the next month.
        # If it is 0x0F, it means that there is no leap month.
        self.G_LUNAR_MONTH_DAYS = (
            0xF0EA4,  # 1901
            0xF1D4A,
            0x52C94,
            0xF0C96,
            0xF1536,
            0x42AAC,
            0xF0AD4,
            0xF16B2,
            0x22EA4,
            0xF0EA4,  # 1910
            0x6364A,
            0xF164A,
            0xF1496,
            0x52956,
            0xF055A,
            0xF0AD6,
            0x216D2,
            0xF1B52,
            0x73B24,
            0xF1D24,  # 1920
            0xF1A4A,
            0x5349A,
            0xF14AC,
            0xF056C,
            0x42B6A,
            0xF0DA8,
            0xF1D52,
            0x23D24,
            0xF1D24,
            0x61A4C,  # 1930
            0xF0A56,
            0xF14AE,
            0x5256C,
            0xF16B4,
            0xF0DA8,
            0x31D92,
            0xF0E92,
            0x72D26,
            0xF1526,
            0xF0A56,  # 1940
            0x614B6,
            0xF155A,
            0xF0AD4,
            0x436AA,
            0xF1748,
            0xF1692,
            0x23526,
            0xF152A,
            0x72A5A,
            0xF0A6C,  # 1950
            0xF155A,
            0x52B54,
            0xF0B64,
            0xF1B4A,
            0x33A94,
            0xF1A94,
            0x8152A,
            0xF152E,
            0xF0AAC,
            0x6156A,  # 1960
            0xF15AA,
            0xF0DA4,
            0x41D4A,
            0xF1D4A,
            0xF0C94,
            0x3192E,
            0xF1536,
            0x72AB4,
            0xF0AD4,
            0xF16D2,  # 1970
            0x52EA4,
            0xF16A4,
            0xF164A,
            0x42C96,
            0xF1496,
            0x82956,
            0xF055A,
            0xF0ADA,
            0x616D2,
            0xF1B52,  # 1980
            0xF1B24,
            0x43A4A,
            0xF1A4A,
            0xA349A,
            0xF14AC,
            0xF056C,
            0x60B6A,
            0xF0DAA,
            0xF1D92,
            0x53D24,  # 1990
            0xF1D24,
            0xF1A4C,
            0x314AC,
            0xF14AE,
            0x829AC,
            0xF06B4,
            0xF0DAA,
            0x52D92,
            0xF0E92,
            0xF0D26,  # 2000
            0x42A56,
            0xF0A56,
            0xF14B6,
            0x22AB4,
            0xF0AD4,
            0x736AA,
            0xF1748,
            0xF1692,
            0x53526,
            0xF152A,  # 2010
            0xF0A5A,
            0x4155A,
            0xF156A,
            0x92B54,
            0xF0BA4,
            0xF1B4A,
            0x63A94,
            0xF1A94,
            0xF192A,
            0x42A5C,  # 2020
            0xF0AAC,
            0xF156A,
            0x22B64,
            0xF0DA4,
            0x61D52,
            0xF0E4A,
            0xF0C96,
            0x5192E,
            0xF1956,
            0xF0AB4,  # 2030
            0x315AC,
            0xF16D2,
            0xB2EA4,
            0xF16A4,
            0xF164A,
            0x63496,
            0xF1496,
            0xF0956,
            0x50AB6,
            0xF0B5A,  # 2040
            0xF16D4,
            0x236A4,
            0xF1B24,
            0x73A4A,
            0xF1A4A,
            0xF14AA,
            0x5295A,
            0xF096C,
            0xF0B6A,
            0x31B54,  # 2050
            0xF1D92,
            0x83D24,
            0xF1D24,
            0xF1A4C,
            0x614AC,
            0xF14AE,
            0xF09AC,
            0x40DAA,
            0xF0EAA,
            0xF0E92,  # 2060
            0x31D26,
            0xF0D26,
            0x72A56,
            0xF0A56,
            0xF14B6,
            0x52AB4,
            0xF0AD4,
            0xF16CA,
            0x42E94,
            0xF1694,  # 2070
            0x8352A,
            0xF152A,
            0xF0A5A,
            0x6155A,
            0xF156A,
            0xF0B54,
            0x4174A,
            0xF1B4A,
            0xF1A94,
            0x3392A,  # 2080
            0xF192C,
            0x7329C,
            0xF0AAC,
            0xF156A,
            0x52B64,
            0xF0DA4,
            0xF1D4A,
            0x41C94,
            0xF0C96,
            0x8192E,  # 2090
            0xF0956,
            0xF0AB6,
            0x615AC,
            0xF16D4,
            0xF0EA4,
            0x42E4A,
            0xF164A,
            0xF1516,
            0x22936,  # 2100
        )
        # Define range of years covered
        self.START_YEAR = 1901
        self.END_YEAR = 2099
        # The 1st day of the 1st month of the Gregorian calendar is 1901/2/19
        self.LUNAR_START_DATE = ((1901, 1, 1),)
        self.SOLAR_START_DATE = date(1901, 2, 19)
        # The Gregorian date for December 30, 2099 is 2100/2/8
        self.LUNAR_END_DATE = (2099, 12, 30)
        self.SOLAR_END_DATE = date(2100, 2, 18)

    @lru_cache
    def _get_leap_month(self, lunar_year: int) -> int:
        """
        Return the number of the leap month if one exists in the year,
        otherwise 15.
        """
        return (self.G_LUNAR_MONTH_DAYS[lunar_year - self.START_YEAR] >> 16) & 0x0F

    def _lunar_month_days(self, lunar_year: int, lunar_month: int) -> int:
        """
        Return the number of days in a lunar month.
        """
        return 29 + ((self.G_LUNAR_MONTH_DAYS[lunar_year - self.START_YEAR] >> lunar_month) & 0x01)

    def _lunar_year_days(self, year: int) -> int:
        """
        Return the number of days in a lunar year.
        """
        days = 0
        months_day = self.G_LUNAR_MONTH_DAYS[year - self.START_YEAR]
        for i in range(1, 13 if self._get_leap_month(year) == 0x0F else 14):
            day = 29 + ((months_day >> i) & 0x01)
            days += day
        return days

    @lru_cache
    def _span_days(self, year: int) -> int:
        """
        Return the number of days elapsed since self.SOLAR_START_DATE to the
        beginning of the year.
        """
        return sum(self._lunar_year_days(y) for y in range(self.START_YEAR, year))

    def lunar_to_gre(self, year: int, month: int, day: int, leap: bool = True) -> date:
        """
        Return the Gregorian date of a Chinese lunar day and month in a
        given Gregorian year.
        """
        leap_month = self._get_leap_month(year) if leap else 15
        span_days = sum(
            self._lunar_month_days(year, m) for m in range(1, month + (month > leap_month))
        )
        span_days += self._span_days(year) + day - 1
        return self.SOLAR_START_DATE + td(days=span_days)

    def vesak_date(self, year: int) -> date:
        """
        Return the estimated Gregorian date of Vesak for Thailand, Laos,
        Singapore and Indonesia, corresponding to 15th day of 4th month
        of the Chinese lunar calendar. See `Wikipedia
        <https://en.wikipedia.org/wiki/Vesak#Dates_of_observance>`__.
        """
        return self.lunar_to_gre(year, 4, 15)

    def vesak_may_date(self, year: int) -> date:
        """
        Return the estimated Gregorian date of Vesak for Sri Lanka, Nepal,
        India, Bangladesh and Malaysia, corresponding to the day of the
        first full moon in May in the Gregorian calendar. See `Wikipedia
        <https://en.wikipedia.org/wiki/Vesak#Dates_of_observance>`__.
        """
        dt = self.lunar_to_gre(year, 3, 15, leap=False)
        return dt if dt.month == 5 else self.lunar_to_gre(year, 4, 15, leap=False)

    def s_diwali_date(self, year: int) -> date:
        """
        Return the estimated Gregorian date of Southern India (Tamil)
        Diwali. Defined as the date of Amāvásyā (new moon) of Kārttikai, which
        corresponds with the months of November or December in the Gregorian
        calendar. See `Wikipedia <https://en.wikipedia.org/wiki/Diwali>`__.
        """
        return self.lunar_to_gre(year, 10, 1) + td(days=-2)

    def thaipusam_date(self, year: int) -> date:
        """
        Return the estimated Gregorian date of Thaipusam (Tamil). Defined as
        the date of the full moon in the Tamil month of Thai, which
        corresponds with the months of January or February in the Gregorian
        calendar. See `Wikipedia <https://en.wikipedia.org/wiki/Thaipusam>`__.
        """
        leap_month = self._get_leap_month(year)
        return self.lunar_to_gre(year, 1 if leap_month <= 6 else 2, 1) + td(days=-15)


CLASS_NAME = "_{cal_name}Lunisolar"
OUT_FILE_NAME = "{cal_name}_dates.py"

CLASS_TEMPLATE = """class {class_name}:
{holiday_data}"""

HOLIDAY_ARRAY_TEMPLATE = """    {hol_name}_DATES = {{
{year_dates}
    }}
"""

YEAR_TEMPLATE = "        {year}: ({date}),"

BUDDHIST, CHINESE, HINDU = range(3)
CALENDARS = {
    BUDDHIST: "Buddhist",
    CHINESE: "Chinese",
    HINDU: "Hindu",
}

ASIAN_HOLIDAYS = (
    # CN, HK, ID, KR, MY, PH, SG, TW, VN
    (1, 1, "LUNAR_NEW_YEAR", CHINESE),  # Lunar New Year
    # VN
    (3, 10, "HUNG_KINGS", CHINESE),  # Kings' Commemoration Day
    # HK, KR
    (4, 8, "BUDDHA_BIRTHDAY", CHINESE),  # Buddha's Birthday
    # ID, SG
    (4, 15, "VESAK", BUDDHIST),  # Vesak
    # CN, HK, TW
    (5, 5, "DRAGON_BOAT", CHINESE),  # Dragon Boat Festival
    # CN, HK, KR, TW
    (8, 15, "MID_AUTUMN", CHINESE),  # Mid-Autumn Festival
    # HK
    (9, 9, "DOUBLE_NINTH", CHINESE),  # Double Ninth Festival
    # MY
    (0, 0, "VESAK_MAY", BUDDHIST),  # Vesak (May)
    # MY, SG
    (0, 0, "DIWALI", HINDU),  # Diwali
    # MY
    (0, 0, "THAIPUSAM", HINDU),  # Thaipusam
)


def generate_data():
    cnls = _Lunisolar()

    g_year_min, g_year_max = (1901, 2099)
    dates = {}
    for g_year in range(g_year_min, g_year_max + 1):
        for h_month, h_day, hol_name, _ in ASIAN_HOLIDAYS:
            if h_month == 0:
                continue
            g_date = cnls.lunar_to_gre(g_year, h_month, h_day)
            if g_year in dates:
                dates[g_year][hol_name] = g_date
            else:
                dates[g_year] = {hol_name: g_date}
        dates[g_year]["DIWALI"] = cnls.s_diwali_date(g_year)
        dates[g_year]["THAIPUSAM"] = cnls.thaipusam_date(g_year)
        dates[g_year]["VESAK_MAY"] = cnls.vesak_may_date(g_year)

    for calendar in CALENDARS:
        holiday_names = sorted(d[2] for d in ASIAN_HOLIDAYS if d[3] == calendar)
        holiday_data = []
        for hol_name in holiday_names:
            year_dates = []
            for year in range(g_year_min, g_year_max + 1):
                d = dates[year].get(hol_name)
                date_str = f"{d.strftime('%b').upper()}, {d.day}"
                year_dates.append(YEAR_TEMPLATE.format(year=year, date=date_str))
            year_dates_str = "\n".join(year_dates)
            holiday_data.append(
                HOLIDAY_ARRAY_TEMPLATE.format(hol_name=hol_name, year_dates=year_dates_str)
            )
        holiday_data_str = "\n".join(holiday_data)
        cal_name = CALENDARS[calendar]
        class_str = CLASS_TEMPLATE.format(
            class_name=CLASS_NAME.format(cal_name=cal_name),
            holiday_data=holiday_data_str,
        )

        path = Path("holidays/calendars") / OUT_FILE_NAME.format(cal_name=cal_name.lower())
        path.write_text(class_str, encoding="UTF-8")


if __name__ == "__main__":
    generate_data()



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.constants import BANK, OPTIONAL, PUBLIC
from holidays.countries.cyprus import Cyprus, CY, CYP
from tests.common import CommonCountryTests


class TestCyprus(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Cyprus, years=range(2000, 2025))

    def test_country_aliases(self):
        self.assertAliases(Cyprus, CY, CYP)

    def test_no_holidays(self):
        self.assertNoHolidays(Cyprus(categories=(BANK, OPTIONAL, PUBLIC), years=1960))

    def test_fixed_holidays(self):
        fdays = (
            (1, 1, "Πρωτοχρονιά"),
            (1, 6, "Ημέρα των Θεοφανίων"),
            (3, 25, "Ημέρα της Ελληνικής Ανεξαρτησίας"),
            (4, 1, "Εθνική Ημέρα Κύπρου"),
            (5, 1, "Πρωτομαγιά"),
            (8, 15, "Κοίμηση της Θεοτόκου"),
            (10, 1, "Ημέρα της Κυπριακής Ανεξαρτησίας"),
            (10, 28, "Ημέρα του Όχι"),
            (12, 25, "Χριστούγεννα"),
            (12, 26, "Επομένη Χριστουγέννων"),
        )
        for y in range(2000, 2025):
            for m, d, name in fdays:
                self.assertHolidayName(name, f"{y}-{m}-{d}")

    def test_green_monday(self):
        self.assertHolidayName(
            "Καθαρά Δευτέρα",
            "2018-02-19",
            "2019-03-11",
            "2020-03-02",
            "2021-03-15",
            "2022-03-07",
            "2023-02-27",
            "2024-03-18",
        )

    def test_good_friday(self):
        self.assertHolidayName(
            "Μεγάλη Παρασκευή",
            "2018-04-06",
            "2019-04-26",
            "2020-04-17",
            "2021-04-30",
            "2022-04-22",
            "2023-04-14",
            "2024-05-03",
        )

    def test_easter_sunday(self):
        self.assertHolidayName(
            "Κυριακή του Πάσχα",
            "2018-04-08",
            "2019-04-28",
            "2020-04-19",
            "2021-05-02",
            "2022-04-24",
            "2023-04-16",
            "2024-05-05",
        )

    def test_easter_monday(self):
        self.assertHolidayName(
            "Δευτέρα της Διακαινησίμου",
            "2018-04-09",
            "2019-04-29",
            "2020-04-20",
            "2021-05-03",
            "2022-04-25",
            "2023-04-17",
            "2024-05-06",
        )

    def test_whit_monday(self):
        self.assertHolidayName(
            "Δευτέρα του Αγίου Πνεύματος",
            "2018-05-28",
            "2019-06-17",
            "2020-06-08",
            "2021-06-21",
            "2022-06-13",
            "2023-06-05",
            "2024-06-24",
        )

    def test_cyprus_independence_day(self):
        name = "Ημέρα της Κυπριακής Ανεξαρτησίας"
        self.assertHolidayName(name, (f"{year}-10-01" for year in range(1979, 2025)))
        self.assertNoHolidayName(name, Cyprus(years=range(1961, 1979)))

    def test_bank_2023(self):
        self.assertHolidays(
            Cyprus(categories=BANK, years=2023),
            ("2023-04-18", "Τρίτη της Διακαινησίμου"),
        )

    def test_optional_2023(self):
        self.assertHolidays(
            Cyprus(categories=OPTIONAL, years=2023),
            ("2023-04-15", "Μεγάλο Σάββατο"),
            ("2023-12-24", "Παραμονή Χριστουγέννων"),
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Πρωτοχρονιά"),
            ("2023-01-06", "Ημέρα των Θεοφανίων"),
            ("2023-02-27", "Καθαρά Δευτέρα"),
            ("2023-03-25", "Ημέρα της Ελληνικής Ανεξαρτησίας"),
            ("2023-04-01", "Εθνική Ημέρα Κύπρου"),
            ("2023-04-14", "Μεγάλη Παρασκευή"),
            ("2023-04-15", "Μεγάλο Σάββατο"),
            ("2023-04-16", "Κυριακή του Πάσχα"),
            ("2023-04-17", "Δευτέρα της Διακαινησίμου"),
            ("2023-04-18", "Τρίτη της Διακαινησίμου"),
            ("2023-05-01", "Πρωτομαγιά"),
            ("2023-06-05", "Δευτέρα του Αγίου Πνεύματος"),
            ("2023-08-15", "Κοίμηση της Θεοτόκου"),
            ("2023-10-01", "Ημέρα της Κυπριακής Ανεξαρτησίας"),
            ("2023-10-28", "Ημέρα του Όχι"),
            ("2023-12-24", "Παραμονή Χριστουγέννων"),
            ("2023-12-25", "Χριστούγεννα"),
            ("2023-12-26", "Επομένη Χριστουγέννων"),
        )

    def test_l10n_en_cy(self):
        self.assertLocalizedHolidays(
            "en_CY",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-06", "Epiphany"),
            ("2023-02-27", "Green Monday"),
            ("2023-03-25", "Greek Independence Day"),
            ("2023-04-01", "Cyprus National Day"),
            ("2023-04-14", "Good Friday"),
            ("2023-04-15", "Holy Saturday"),
            ("2023-04-16", "Easter Sunday"),
            ("2023-04-17", "Easter Monday"),
            ("2023-04-18", "Easter Tuesday"),
            ("2023-05-01", "Labour Day"),
            ("2023-06-05", "Pentecost"),
            ("2023-08-15", "Assumption Day"),
            ("2023-10-01", "Cyprus Independence Day"),
            ("2023-10-28", "Ochi Day"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Day After Christmas"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-06", "Epiphany"),
            ("2023-02-27", "Green Monday"),
            ("2023-03-25", "Greek Independence Day"),
            ("2023-04-01", "Cyprus National Day"),
            ("2023-04-14", "Good Friday"),
            ("2023-04-15", "Holy Saturday"),
            ("2023-04-16", "Easter Sunday"),
            ("2023-04-17", "Easter Monday"),
            ("2023-04-18", "Easter Tuesday"),
            ("2023-05-01", "Labor Day"),
            ("2023-06-05", "Whit Monday"),
            ("2023-08-15", "Assumption Day"),
            ("2023-10-01", "Cyprus Independence Day"),
            ("2023-10-28", "Greek National Day"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Day After Christmas"),
        )

    def test_l10n_uk(self):
        self.assertLocalizedHolidays(
            "uk",
            ("2023-01-01", "Новий рік"),
            ("2023-01-06", "Богоявлення"),
            ("2023-02-27", "Чистий понеділок"),
            ("2023-03-25", "День незалежності Греції"),
            ("2023-04-01", "Національне свято Кіпру"),
            ("2023-04-14", "Страсна пʼятниця"),
            ("2023-04-15", "Велика субота"),
            ("2023-04-16", "Великдень"),
            ("2023-04-17", "Великодній понеділок"),
            ("2023-04-18", "Великодній вівторок"),
            ("2023-05-01", "День праці"),
            ("2023-06-05", "День Святого Духа"),
            ("2023-08-15", "Успіння Пресвятої Богородиці"),
            ("2023-10-01", "День незалежності Кіпру"),
            ("2023-10-28", "День Охі"),
            ("2023-12-24", "Святий вечір"),
            ("2023-12-25", "Різдво Христове"),
            ("2023-12-26", "Другий день Різдва"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.san_marino import SanMarino, SM, SMR
from tests.common import CommonCountryTests


class TestSanMarino(CommonCountryTests, TestCase):
    def setUp(self):
        self.holidays = SanMarino()

    def test_country_aliases(self):
        self.assertAliases(SanMarino, SM, SMR)

    def test_2022(self):
        self.assertHolidays(
            ("2022-01-01", "New Year's Day"),
            ("2022-01-06", "Epiphany"),
            ("2022-02-05", "Feast of Saint Agatha"),
            ("2022-03-25", "Anniversary of the Arengo"),
            ("2022-04-17", "Easter Sunday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-05-01", "Labour Day"),
            ("2022-06-16", "Corpus Cristi"),
            ("2022-07-28", "Liberation from Fascism Day"),
            ("2022-08-15", "Assumption Day"),
            ("2022-09-03", "Foundation Day"),
            ("2022-11-01", "All Saints' Day"),
            ("2022-11-02", "Commemoration of the Dead"),
            ("2022-12-08", "Immaculate Conception Day"),
            ("2022-12-24", "Christmas Eve"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Saint Stephen's Day"),
            ("2022-12-31", "New Year's Eve"),
        )

    def test_2023(self):
        self.assertHolidays(
            ("2023-01-01", "New Year's Day"),
            ("2023-01-06", "Epiphany"),
            ("2023-02-05", "Feast of Saint Agatha"),
            ("2023-03-25", "Anniversary of the Arengo"),
            ("2023-04-09", "Easter Sunday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-01", "Labour Day"),
            ("2023-06-08", "Corpus Cristi"),
            ("2023-07-28", "Liberation from Fascism Day"),
            ("2023-08-15", "Assumption Day"),
            ("2023-09-03", "Foundation Day"),
            ("2023-11-01", "All Saints' Day"),
            ("2023-11-02", "Commemoration of the Dead"),
            ("2023-12-08", "Immaculate Conception Day"),
            ("2023-12-24", "Christmas Eve"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Saint Stephen's Day"),
            ("2023-12-31", "New Year's Eve"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.brunei import Brunei, BN, BRN
from tests.common import CommonCountryTests


class TestBrunei(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Brunei, years=range(1984, 2077), years_non_observed=range(1984, 2077))

    def test_country_aliases(self):
        self.assertAliases(Brunei, BN, BRN)

    def test_no_holidays(self):
        self.assertNoHolidays(Brunei(years=1983))

    def test_special_holidays(self):
        self.assertHoliday(
            "1998-08-10",
            "2004-09-09",
            "2017-10-05",
        )

    def test_2022(self):
        self.assertHolidays(
            Brunei(years=2022),
            ("2022-01-01", "Awal Tahun Masihi"),
            ("2022-02-01", "Tahun Baru Cina"),
            ("2022-02-23", "Hari Kebangsaan"),
            ("2022-02-28", "Israk dan Mikraj"),
            ("2022-04-03", "Hari Pertama Berpuasa"),
            ("2022-04-04", "Hari Pertama Berpuasa (diperhatikan)"),
            ("2022-04-19", "Hari Nuzul Al-Quran"),
            ("2022-05-02", "Hari Raya Aidil Fitri"),
            ("2022-05-03", "Hari Raya Aidil Fitri"),
            ("2022-05-04", "Hari Raya Aidil Fitri"),
            ("2022-05-31", "Hari Angkatan Bersenjata Diraja Brunei"),
            ("2022-07-09", "Hari Raya Aidil Adha"),
            ("2022-07-15", "Hari Keputeraan KDYMM Sultan Brunei"),
            ("2022-07-16", "Hari Keputeraan KDYMM Sultan Brunei (diperhatikan)"),
            ("2022-07-30", "Awal Tahun Hijrah"),
            ("2022-10-08", "Maulidur Rasul"),
            ("2022-12-25", "Hari Natal"),
            ("2022-12-26", "Hari Natal (diperhatikan)"),
        )

    def test_2023(self):
        self.assertHolidays(
            Brunei(years=2023),
            ("2023-01-01", "Awal Tahun Masihi"),
            ("2023-01-02", "Awal Tahun Masihi (diperhatikan)"),
            ("2023-01-22", "Tahun Baru Cina"),
            ("2023-01-23", "Tahun Baru Cina (diperhatikan)"),
            ("2023-02-18", "Israk dan Mikraj"),
            ("2023-02-23", "Hari Kebangsaan"),
            ("2023-03-23", "Hari Pertama Berpuasa"),
            ("2023-04-08", "Hari Nuzul Al-Quran"),
            ("2023-04-22", "Hari Raya Aidil Fitri"),
            ("2023-04-23", "Hari Raya Aidil Fitri"),
            ("2023-04-24", "Hari Raya Aidil Fitri"),
            ("2023-04-25", "Hari Raya Aidil Fitri (diperhatikan)"),
            ("2023-05-31", "Hari Angkatan Bersenjata Diraja Brunei"),
            ("2023-06-29", "Hari Raya Aidil Adha"),
            ("2023-07-15", "Hari Keputeraan KDYMM Sultan Brunei"),
            ("2023-07-19", "Awal Tahun Hijrah"),
            ("2023-09-28", "Maulidur Rasul"),
            ("2023-12-25", "Hari Natal"),
        )

    def test_2024(self):
        self.assertHolidays(
            Brunei(years=2024),
            ("2024-01-01", "Awal Tahun Masihi"),
            ("2024-02-08", "Israk dan Mikraj"),
            ("2024-02-10", "Tahun Baru Cina"),
            ("2024-02-23", "Hari Kebangsaan"),
            ("2024-02-24", "Hari Kebangsaan (diperhatikan)"),
            ("2024-03-12", "Hari Pertama Berpuasa"),
            ("2024-03-28", "Hari Nuzul Al-Quran"),
            ("2024-04-10", "Hari Raya Aidil Fitri"),
            ("2024-04-11", "Hari Raya Aidil Fitri"),
            ("2024-04-12", "Hari Raya Aidil Fitri"),
            ("2024-04-13", "Hari Raya Aidil Fitri (diperhatikan)"),
            ("2024-05-31", "Hari Angkatan Bersenjata Diraja Brunei"),
            ("2024-06-01", "Hari Angkatan Bersenjata Diraja Brunei (diperhatikan)"),
            ("2024-06-17", "Hari Raya Aidil Adha"),
            ("2024-07-07", "Awal Tahun Hijrah"),
            ("2024-07-08", "Awal Tahun Hijrah (diperhatikan)"),
            ("2024-07-15", "Hari Keputeraan KDYMM Sultan Brunei"),
            ("2024-09-16", "Maulidur Rasul"),
            ("2024-12-25", "Hari Natal"),
        )

    def test_2025(self):
        self.assertHolidays(
            Brunei(years=2025),
            ("2025-01-01", "Awal Tahun Masihi"),
            ("2025-01-27", "Israk dan Mikraj"),
            ("2025-01-29", "Tahun Baru Cina"),
            ("2025-02-23", "Hari Kebangsaan"),
            ("2025-02-24", "Hari Kebangsaan (diperhatikan)"),
            ("2025-03-02", "Hari Pertama Berpuasa"),
            ("2025-03-03", "Hari Pertama Berpuasa (diperhatikan)"),
            ("2025-03-18", "Hari Nuzul Al-Quran"),
            ("2025-03-31", "Hari Raya Aidil Fitri"),
            ("2025-04-01", "Hari Raya Aidil Fitri"),
            ("2025-04-02", "Hari Raya Aidil Fitri"),
            ("2025-05-31", "Hari Angkatan Bersenjata Diraja Brunei"),
            ("2025-06-07", "Hari Raya Aidil Adha"),
            ("2025-06-27", "Awal Tahun Hijrah"),
            ("2025-06-28", "Awal Tahun Hijrah (diperhatikan)"),
            ("2025-07-15", "Hari Keputeraan KDYMM Sultan Brunei"),
            ("2025-09-05", "Maulidur Rasul"),
            ("2025-09-06", "Maulidur Rasul (diperhatikan)"),
            ("2025-12-25", "Hari Natal"),
        )

    def test_national_day(self):
        self.assertHoliday(f"{year}-02-23" for year in range(1984, 2077))

        self.assertNoNonObservedHoliday(
            "2001-02-24",
            "2003-02-24",
            "2007-02-24",
            "2014-02-24",
            "2018-02-24",
            "2020-02-24",
            "2024-02-24",
            "2025-02-24",
        )

    def test_armed_forces_day(self):
        self.assertHoliday(f"{year}-05-31" for year in range(1984, 2077))

        self.assertNoNonObservedHoliday(
            "2002-06-01",
            "2009-06-01",
            "2013-06-01",
            "2015-06-01",
            "2019-06-01",
            "2020-06-01",
            "2024-06-01",
        )

    def test_sultan_hassanal_bolkiah_birthday(self):
        self.assertHoliday(f"{year}-07-15" for year in range(1984, 2077))

        self.assertNoNonObservedHoliday(
            "2001-07-16",
            "2005-07-16",
            "2007-07-16",
            "2011-07-16",
            "2012-07-16",
            "2016-07-16",
            "2018-07-16",
            "2022-07-16",
        )

    def test_l10n_default(self):
        self.assertLocalizedHolidays(
            ("2023-01-01", "Awal Tahun Masihi"),
            ("2023-01-02", "Awal Tahun Masihi (diperhatikan)"),
            ("2023-01-22", "Tahun Baru Cina"),
            ("2023-01-23", "Tahun Baru Cina (diperhatikan)"),
            ("2023-02-18", "Israk dan Mikraj"),
            ("2023-02-23", "Hari Kebangsaan"),
            ("2023-03-23", "Hari Pertama Berpuasa"),
            ("2023-04-08", "Hari Nuzul Al-Quran"),
            ("2023-04-22", "Hari Raya Aidil Fitri"),
            ("2023-04-23", "Hari Raya Aidil Fitri"),
            ("2023-04-24", "Hari Raya Aidil Fitri"),
            ("2023-04-25", "Hari Raya Aidil Fitri (diperhatikan)"),
            ("2023-05-31", "Hari Angkatan Bersenjata Diraja Brunei"),
            ("2023-06-29", "Hari Raya Aidil Adha"),
            ("2023-07-15", "Hari Keputeraan KDYMM Sultan Brunei"),
            ("2023-07-19", "Awal Tahun Hijrah"),
            ("2023-09-28", "Maulidur Rasul"),
            ("2023-12-25", "Hari Natal"),
        )

    def test_l10n_en_us(self):
        self.assertLocalizedHolidays(
            "en_US",
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-22", "Lunar New Year"),
            ("2023-01-23", "Lunar New Year (observed)"),
            ("2023-02-18", "Isra' and Mi'raj"),
            ("2023-02-23", "National Day"),
            ("2023-03-23", "First Day of Ramadan"),
            ("2023-04-08", "Anniversary of the revelation of the Quran"),
            ("2023-04-22", "Eid al-Fitr"),
            ("2023-04-23", "Eid al-Fitr"),
            ("2023-04-24", "Eid al-Fitr"),
            ("2023-04-25", "Eid al-Fitr (observed)"),
            ("2023-05-31", "Armed Forces Day"),
            ("2023-06-29", "Eid al-Adha"),
            ("2023-07-15", "Sultan Hassanal Bolkiah's Birthday"),
            ("2023-07-19", "Islamic New Year"),
            ("2023-09-28", "Prophet's Birthday"),
            ("2023-12-25", "Christmas Day"),
        )

    def test_l10n_th(self):
        self.assertLocalizedHolidays(
            "th",
            ("2023-01-01", "วันขึ้นปีใหม่"),
            ("2023-01-02", "ชดเชยวันขึ้นปีใหม่"),
            ("2023-01-22", "วันตรุษจีน"),
            ("2023-01-23", "ชดเชยวันตรุษจีน"),
            ("2023-02-18", "วันเมี๊ยะราจ"),
            ("2023-02-23", "วันชาติบรูไน"),
            ("2023-03-23", "วันแรกการถือศีลอด"),
            ("2023-04-08", "วันนูซุลอัลกุรอาน"),
            ("2023-04-22", "วันอีฎิ้ลฟิตริ"),
            ("2023-04-23", "วันอีฎิ้ลฟิตริ"),
            ("2023-04-24", "วันอีฎิ้ลฟิตริ"),
            ("2023-04-25", "ชดเชยวันอีฎิ้ลฟิตริ"),
            ("2023-05-31", "วันกองทัพบรูไน"),
            ("2023-06-29", "วันอีดิ้ลอัฎฮา"),
            ("2023-07-15", "วันเฉลิมพระชนมพรรษาสมเด็จพระราชาธิบดีสุลต่านฮัสซานัล โบลเกียห์"),
            ("2023-07-19", "วันขึ้นปีใหม่อิสลาม"),
            ("2023-09-28", "วันเมาลิดนบี"),
            ("2023-12-25", "วันคริสต์มาส"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

from datetime import date
from typing import Dict, List, Union

from docutils import nodes
from docutils.nodes import Element, Node

from sphinx.writers.html import HTMLTranslator

# below required for local build
import sphinx_rtd_theme  # noqa: F401 'sphinx_rtd_theme' imported but unused

sys.path.insert(0, os.path.abspath("../.."))
import holidays  # noqa: E402 module level import not at top of file


# -- Project information -----------------------------------------------------

project = "holidays"
copyright = str(date.today().year)
author = "Vacanza Team"
version = holidays.__version__


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
#
extensions = [
    # 'sphinx.ext.intersphinx',
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    # 'sphinx.ext.doctest',
    # 'sphinx.ext.mathjax',
    "sphinx.ext.todo",
    # 'sphinx.ext.viewcode',
    # 'sphinxcontrib.httpdomain',
    "sphinx_rtd_theme",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns: list[str] = []

# Add any paths that contain templates here, relative to this directory.
#
# templates_path = ['_templates']

# The name of the Pygments (syntax highlighting) style to use.
# If not set, either the theme's default style or 'sphinx' is selected for HTML
# output.
#
pygments_style = "sphinx"

# If your documentation needs a minimal Sphinx version, state it here.
#
needs_sphinx = "4.3.2"

# -- Options for internationalization ----------------------------------------

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages. See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'  # default
html_theme = "sphinx_rtd_theme"

# Custom style (imports regular style and adding new ones)
#
# See https://docs.readthedocs.io/en/stable/guides/adding-custom-css.html
#     #overriding-or-replacing-a-theme-s-stylesheet
#
html_style = "css/custom.css"

# The “title” for HTML documentation generated with Sphinx's own templates.
#
# html_title = '<project> v<revision> documentation'  # default


# Name of image file that is the logo of the docs, or URL that points to same.
# Width not to exceed 200 pixels.
#
# html_logo = None  # default


# Name of a Windows-style icon file (.ico) that is the favicon of the docs, or
# URL that points to same.
#
# html_favicon = None  # default

# A 'Last updated on:' timestamp is inserted at every page bottom, using the
#  given strftime() format. The empty # string is equivalent to '%b %d, %Y'
#  (or a locale-dependent equivalent).
#
html_last_updated_fmt = ""

# Include reST sources in the HTML build as _sources/name. The default is True
#
html_copy_source = False

# Theme options are theme-specific and customize the look and feel of a theme
# further. For a list of options available for each theme, see the
# documentation.
#
# See https://sphinx-rtd-theme.readthedocs.io/en/latest/configuring.html
#
html_theme_options: dict[str, Union[str, bool, int]] = {
    # 'canonical_url': '',
    # 'analytics_id': 'UA-XXXXXXX-1',  #  Provided by Google in your dashboard
    # 'analytics_anonymize_ip': False
    # 'logo_only': False,
    # 'display_version': True,
    # 'prev_next_buttons_location': 'bottom',
    # 'style_external_links': False,
    # 'vcs_pageview_mode': '',
    # 'style_nav_header_background': '#2980B9',
    # Toc options
    # 'collapse_navigation': True,
    # 'sticky_navigation': True,
    # 'navigation_depth': 4,
    # 'includehidden': True,
    # 'titles_only': False
}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named 'default.css' will overwrite the builtin 'default.css'.
html_static_path = ["_static"]

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# The default sidebars (for documents that don't match any pattern) are
# defined by theme itself. Builtin themes are using these templates by
# default: ``['localtoc.html', 'relations.html', 'sourcelink.html',
# 'searchbox.html']``.
#
# html_sidebars = {}


# -- Options for the Python domain -------------------------------------------

# Set to True to suppress the module name of the python reference if it can be
# resolved.
#
python_use_unqualified_type_names = True


# -- sphinx.ext.autodoc configuration ----------------------------------------

# See https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html
#     #configuration

autoclass_content = "both"
autodoc_member_order = "groupwise"
# See https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html
#     #directives
autodoc_default_options = {
    "members": True,
    "member-order": "bysource",  # 'alphabetical' (def) 'groupwise', 'bysource'
    "undoc-members": True,
    # 'private-members': False,  # default
    # 'special-members': False,  # default
    "inherited-members": False,  # Disabled due to performance concerns.
    "show-inheritance": True,
    # 'ignore-module-all': False,  # default
    # 'imported-members': False, # default
    # 'exclude-members': '',  # default
    # 'class-doc-from': '',  # default
}
# autodoc_docstring_signature = True  # default
# autodoc_mock_imports = []  # default
autodoc_typehints = "description"
# autodoc_typehints_description_target = 'all'  # default
# autodoc_type_aliases = {}  # default
# autodoc_warningiserror = True  # default
# autodoc_inherit_docstrings = True  # default


# https://stackoverflow.com/questions/2701998
autosummary_generate = True


# -- sphinx.ext.to do configuration ------------------------------------------

# If true, `to do` and `todoList` produce output, else they produce nothing.
#
todo_include_todos = True


# -- Patch to open external links in new tab ---------------------------------

# see https://stackoverflow.com/a/61669375/3159288

# code below from version 4.0.0 at
# https://github.com/sphinx-doc/sphinx/blob/master/sphinx/writers/html5.py


class PatchedHTMLTranslator(HTMLTranslator):
    """Adds open-in-new-tab support.
    See https://stackoverflow.com/questions/25583581.
    """

    def unknown_visit(self, node: Node) -> None:
        pass

    def visit_reference(self, node: Element) -> None:
        atts = {"class": "reference"}
        if node.get("internal") or "refuri" not in node:
            atts["class"] += " internal"
        else:
            atts["class"] += " external"
            # ---------------------------------------------------------
            # Customize behavior (open in new tab, secure linking site)
            atts["target"] = "_blank"
            atts["rel"] = "noopener noreferrer"
            # ---------------------------------------------------------
        if "refuri" in node:
            atts["href"] = node["refuri"] or "#"
            if self.settings.cloak_email_addresses and atts["href"].startswith(
                "mailto:"
            ):
                atts["href"] = self.cloak_mailto(atts["href"])
                self.in_mailto = True
        else:
            assert (
                "refid" in node
            ), 'References must have "refuri" or "refid" attribute.'
            atts["href"] = "#" + node["refid"]
        if not isinstance(node.parent, nodes.TextElement):
            assert len(node) == 1 and isinstance(node[0], nodes.image)
            atts["class"] += " image-reference"
        if "reftitle" in node:
            atts["title"] = node["reftitle"]
        if "target" in node:
            atts["target"] = node["target"]
        self.body.append(self.starttag(node, "a", "", **atts))

        if node.get("secnumber"):
            self.body.append(
                ("%s" + self.secnumber_suffix)
                % ".".join(map(str, node["secnumber"]))
            )


def setup(app):
    app.set_translator("html", PatchedHTMLTranslator)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from collections.abc import Iterable
from datetime import date
from typing import Optional

from holidays.calendars.custom import _CustomCalendar
from holidays.calendars.gregorian import JAN, FEB, MAR, APR, MAY, JUN, JUL, AUG, SEP, OCT, NOV, DEC
from holidays.helpers import _normalize_tuple

BAK_POYA = "BAK_POYA"
BINARA_POYA = "BINARA_POYA"
DURUTHU_POYA = "DURUTHU_POYA"
ESALA_POYA = "ESALA_POYA"
IL_POYA = "IL_POYA"
MEDIN_POYA = "MEDIN_POYA"
NAWAM_POYA = "NAWAM_POYA"
NIKINI_POYA = "NIKINI_POYA"
POSON_POYA = "POSON_POYA"
UNDUVAP_POYA = "UNDUVAP_POYA"
VAP_POYA = "VAP_POYA"
VESAK_POYA = "VESAK_POYA"


class _SinhalaLunar:
    """
    Sinhala Lunar calendar for 2003-2025 years.

    Their Buddhist Uposatha day calculation method is different from Thai LuniSolar
    and Buddhist (Mahayana) used in East Asia.

    Due to the fact that Poya (Uposatha) days are calculated astronomically
    based on how close a particular day is closest to full moon at noon, and that
    an extra month is added every 33 months interval, this is hardcoded for now.

    Adhi month dates are instead hardcoded in Sri Lanka country implementation.
    """

    START_YEAR = 2003
    END_YEAR = 2025

    BAK_POYA_DATES = {
        2003: (APR, 16),
        2004: (APR, 5),
        2005: (APR, 23),
        2006: (APR, 13),
        2007: (APR, 2),
        2008: (APR, 19),
        2009: (APR, 9),
        2010: (MAR, 29),
        2011: (APR, 17),
        2012: (APR, 6),
        2013: (APR, 25),
        2014: (APR, 14),
        2015: (APR, 3),
        2016: (APR, 21),
        2017: (APR, 10),
        2018: (MAR, 31),
        2019: (APR, 19),
        2020: (APR, 7),
        2021: (APR, 26),
        2022: (APR, 16),
        2023: (APR, 5),
        2024: (APR, 23),
        2025: (APR, 12),
    }

    BINARA_POYA_DATES = {
        2003: (SEP, 10),
        2004: (SEP, 28),
        2005: (SEP, 17),
        2006: (SEP, 7),
        2007: (SEP, 26),
        2008: (SEP, 14),
        2009: (SEP, 4),
        2010: (SEP, 22),
        2011: (SEP, 11),
        2012: (SEP, 29),
        2013: (SEP, 19),
        2014: (SEP, 8),
        2015: (SEP, 27),
        2016: (SEP, 16),
        2017: (SEP, 5),
        2018: (SEP, 24),
        2019: (SEP, 13),
        2020: (SEP, 1),
        2021: (SEP, 20),
        2022: (SEP, 10),
        2023: (SEP, 29),
        2024: (SEP, 17),
        2025: (SEP, 7),
    }

    DURUTHU_POYA_DATES = {
        2003: (JAN, 17),
        2004: (JAN, 7),
        2005: (JAN, 24),
        2006: (JAN, 13),
        2007: (JAN, 3),
        2008: (JAN, 22),
        2009: ((JAN, 10), (DEC, 31)),
        2011: (JAN, 19),
        2012: (JAN, 8),
        2013: (JAN, 26),
        2014: (JAN, 15),
        2015: (JAN, 4),
        2016: (JAN, 23),
        2017: (JAN, 12),
        2018: (JAN, 1),
        2019: (JAN, 20),
        2020: (JAN, 10),
        2021: (JAN, 28),
        2022: (JAN, 17),
        2023: (JAN, 6),
        2024: (JAN, 25),
        2025: (JAN, 13),
    }

    ESALA_POYA_DATES = {
        2003: (JUL, 13),
        2004: (JUL, 2),
        2005: (JUL, 21),
        2006: (JUL, 10),
        2007: (JUL, 29),
        2008: (JUL, 17),
        2009: (JUL, 6),
        2010: (JUL, 25),
        2011: (JUL, 14),
        2012: (JUL, 3),
        2013: (JUL, 22),
        2014: (JUL, 12),
        2015: (JUL, 31),
        2016: (JUL, 19),
        2017: (JUL, 8),
        2018: (JUL, 27),
        2019: (JUL, 16),
        2020: (JUL, 4),
        2021: (JUL, 23),
        2022: (JUL, 13),
        2023: (AUG, 1),
        2024: (JUL, 20),
        2025: (JUL, 10),
    }

    IL_POYA_DATES = {
        2003: (NOV, 8),
        2004: (NOV, 26),
        2005: (NOV, 15),
        2006: (NOV, 5),
        2007: (NOV, 24),
        2008: (NOV, 12),
        2009: (NOV, 2),
        2010: (NOV, 21),
        2011: (NOV, 10),
        2012: (NOV, 27),
        2013: (NOV, 17),
        2014: (NOV, 6),
        2015: (NOV, 25),
        2016: (NOV, 14),
        2017: (NOV, 3),
        2018: (NOV, 22),
        2019: (NOV, 12),
        2020: (NOV, 29),
        2021: (NOV, 18),
        2022: (NOV, 7),
        2023: (NOV, 26),
        2024: (NOV, 15),
        2025: (NOV, 5),
    }

    MEDIN_POYA_DATES = {
        2003: (MAR, 18),
        2004: (MAR, 6),
        2005: (MAR, 25),
        2006: (MAR, 14),
        2007: (MAR, 3),
        2008: (MAR, 21),
        2009: (MAR, 10),
        2010: (FEB, 28),
        2011: (MAR, 19),
        2012: (MAR, 7),
        2013: (MAR, 26),
        2014: (MAR, 16),
        2015: (MAR, 5),
        2016: (MAR, 22),
        2017: (MAR, 12),
        2018: (MAR, 1),
        2019: (MAR, 20),
        2020: (MAR, 9),
        2021: (MAR, 28),
        2022: (MAR, 17),
        2023: (MAR, 6),
        2024: (MAR, 24),
        2025: (MAR, 13),
    }

    NAWAM_POYA_DATES = {
        2003: (FEB, 16),
        2004: (FEB, 5),
        2005: (FEB, 23),
        2006: (FEB, 12),
        2007: (FEB, 1),
        2008: (FEB, 20),
        2009: (FEB, 9),
        2010: (JAN, 29),
        2011: (FEB, 17),
        2012: (FEB, 7),
        2013: (FEB, 25),
        2014: (FEB, 14),
        2015: (FEB, 3),
        2016: (FEB, 22),
        2017: (FEB, 10),
        2018: (JAN, 31),
        2019: (FEB, 19),
        2020: (FEB, 8),
        2021: (FEB, 26),
        2022: (FEB, 16),
        2023: (FEB, 5),
        2024: (FEB, 23),
        2025: (FEB, 12),
    }

    NIKINI_POYA_DATES = {
        2003: (AUG, 11),
        2004: (AUG, 29),
        2005: (AUG, 19),
        2006: (AUG, 9),
        2007: (AUG, 28),
        2008: (AUG, 16),
        2009: (AUG, 5),
        2010: (AUG, 24),
        2011: (AUG, 13),
        2012: (AUG, 1),
        2013: (AUG, 20),
        2014: (AUG, 10),
        2015: (AUG, 29),
        2016: (AUG, 17),
        2017: (AUG, 7),
        2018: (AUG, 25),
        2019: (AUG, 14),
        2020: (AUG, 3),
        2021: (AUG, 22),
        2022: (AUG, 11),
        2023: (AUG, 30),
        2024: (AUG, 19),
        2025: (AUG, 8),
    }

    POSON_POYA_DATES = {
        2003: (JUN, 14),
        2004: (JUN, 2),
        2005: (JUN, 21),
        2006: (JUN, 11),
        2007: (JUN, 30),
        2008: (JUN, 18),
        2009: (JUN, 7),
        2010: (JUN, 25),
        2011: (JUN, 15),
        2012: (JUN, 4),
        2013: (JUN, 23),
        2014: (JUN, 12),
        2015: (JUN, 2),
        2016: (JUN, 19),
        2017: (JUN, 8),
        2018: (JUN, 27),
        2019: (JUN, 16),
        2020: (JUN, 5),
        2021: (JUN, 24),
        2022: (JUN, 14),
        2023: (JUN, 3),
        2024: (JUN, 21),
        2025: (JUN, 10),
    }

    UNDUVAP_POYA_DATES = {
        2003: (DEC, 8),
        2004: (DEC, 26),
        2005: (DEC, 15),
        2006: (DEC, 4),
        2007: (DEC, 23),
        2008: (DEC, 12),
        2009: (DEC, 1),
        2010: (DEC, 20),
        2011: (DEC, 10),
        2012: (DEC, 27),
        2013: (DEC, 16),
        2014: (DEC, 6),
        2015: (DEC, 24),
        2016: (DEC, 13),
        2017: (DEC, 3),
        2018: (DEC, 22),
        2019: (DEC, 11),
        2020: (DEC, 29),
        2021: (DEC, 18),
        2022: (DEC, 7),
        2023: (DEC, 26),
        2024: (DEC, 14),
        2025: (DEC, 4),
    }

    VAP_POYA_DATES = {
        2003: (OCT, 9),
        2004: (OCT, 27),
        2005: (OCT, 17),
        2006: (OCT, 6),
        2007: (OCT, 25),
        2008: (OCT, 14),
        2009: (OCT, 3),
        2010: (OCT, 22),
        2011: (OCT, 11),
        2012: (OCT, 29),
        2013: (OCT, 18),
        2014: (OCT, 8),
        2015: (OCT, 27),
        2016: (OCT, 15),
        2017: (OCT, 5),
        2018: (OCT, 24),
        2019: (OCT, 13),
        2020: (OCT, 30),
        2021: (OCT, 20),
        2022: (OCT, 9),
        2023: (OCT, 28),
        2024: (OCT, 17),
        2025: (OCT, 6),
    }

    VESAK_POYA_DATES = {
        2003: (MAY, 15),
        2004: (MAY, 4),
        2005: (MAY, 23),
        2006: (MAY, 12),
        2007: (MAY, 1),
        2008: (MAY, 19),
        2009: (MAY, 8),
        2010: (MAY, 27),
        2011: (MAY, 17),
        2012: (MAY, 5),
        2013: (MAY, 24),
        2014: (MAY, 14),
        2015: (MAY, 3),
        2016: (MAY, 21),
        2017: (MAY, 10),
        2018: (APR, 29),
        2019: (MAY, 18),
        2020: (MAY, 7),
        2021: (MAY, 26),
        2022: (MAY, 15),
        2023: (MAY, 5),
        2024: (MAY, 23),
        2025: (MAY, 12),
    }

    def _get_holiday(self, holiday: str, year: int) -> tuple[Optional[date], bool]:
        estimated_dates = getattr(self, f"{holiday}_DATES", {})
        exact_dates = getattr(self, f"{holiday}_DATES_{_CustomCalendar.CUSTOM_ATTR_POSTFIX}", {})
        dt = exact_dates.get(year, estimated_dates.get(year, ()))
        return date(year, *dt) if dt else None, year not in exact_dates

    def _get_holiday_set(self, holiday: str, year: int) -> Iterable[tuple[date, bool]]:
        estimated_dates = getattr(self, f"{holiday}_DATES", {})
        exact_dates = getattr(self, f"{holiday}_DATES_{_CustomCalendar.CUSTOM_ATTR_POSTFIX}", {})
        for year in (year - 1, year):
            for dt in _normalize_tuple(exact_dates.get(year, estimated_dates.get(year, ()))):
                yield date(year, *dt), year not in exact_dates

    def bak_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(BAK_POYA, year)

    def binara_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(BINARA_POYA, year)

    def duruthu_poya_date(self, year: int) -> Iterable[tuple[date, bool]]:
        return self._get_holiday_set(DURUTHU_POYA, year)

    def esala_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(ESALA_POYA, year)

    def il_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(IL_POYA, year)

    def medin_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(MEDIN_POYA, year)

    def nawam_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(NAWAM_POYA, year)

    def nikini_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(NIKINI_POYA, year)

    def poson_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(POSON_POYA, year)

    def unduvap_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(UNDUVAP_POYA, year)

    def vap_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(VAP_POYA, year)

    def vesak_poya_date(self, year: int) -> tuple[Optional[date], bool]:
        return self._get_holiday(VESAK_POYA, year)


class _CustomSinhalaHolidays(_CustomCalendar, _SinhalaLunar):
    pass



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.south_africa import SouthAfrica, ZA, ZAF
from tests.common import CommonCountryTests


class TestSouthAfrica(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(SouthAfrica, years=range(1940, 2050))

    def test_country_aliases(self):
        self.assertAliases(SouthAfrica, ZA, ZAF)

    def test_no_holidays(self):
        self.assertNoHolidays(SouthAfrica(years=1909))

    def test_special_holidays(self):
        self.assertHoliday(
            "1999-06-02",
            "1999-12-31",
            "2000-01-02",
            "2004-04-14",
            "2006-03-01",
            "2008-05-02",
            "2009-04-22",
            "2011-05-18",
            "2011-12-27",
            "2014-05-07",
            "2016-08-03",
            "2019-05-08",
            "2021-11-01",
            "2022-12-27",
            "2023-12-15",
            # presidential
            "2008-05-02",
            "2011-12-27",
            "2016-12-27",
            # elections
            "1999-06-02",  # Election Day 1999
            "2004-04-14",  # Election Day 2004
            "2006-03-01",  # Local Election
            "2009-04-22",  # Election Day 2008
            "2011-05-18",  # Election Day 2011
            "2014-05-07",  # Election Day 2014
            "2016-08-03",  # Election Day 2016
            "2019-05-08",  # Election Day 2019
            "2021-11-01",  # Election Day 2019
            "2024-05-29",  # Election Day 2024
        )

    def test_new_years_day(self):
        self.assertHolidayName("New Year's Day", (f"{year}-01-01" for year in range(1940, 2050)))

    def test_holidays_from_1995(self):
        for year in range(1995, 2050):
            self.assertHoliday(
                f"{year}-03-21",
                f"{year}-04-27",
                f"{year}-05-01",
                f"{year}-06-16",
                f"{year}-08-09",
                f"{year}-09-24",
            )

    def test_easter(self):
        self.assertHolidayName(
            "Good Friday",
            "2018-03-30",
            "2019-04-19",
            "2020-04-10",
            "2021-04-02",
            "2022-04-15",
        )
        self.assertHolidayName(
            "Family Day",
            "2018-04-02",
            "2019-04-22",
            "2020-04-13",
            "2021-04-05",
            "2022-04-18",
        )
        self.assertHolidayName(
            "Easter Monday",
            "1940-03-25",
            "1960-04-18",
            "1970-03-30",
            "1978-03-27",
            "1979-04-16",
        )
        self.assertNoHolidayName("Family Day", range(1940, 1961), range(1974, 1980))
        self.assertNoHolidayName("Easter Monday", range(1980, 2050))

    def test_day_of_reconciliation(self):
        self.assertHoliday(f"{year}-12-16" for year in range(1940, 2050))
        self.assertNoHolidayName("Dingaan's Day", range(1952, 2050))
        self.assertNoHolidayName("Day of the Covenant", range(1940, 1952), range(1980, 2050))
        self.assertNoHolidayName("Day of the Vow", range(1940, 1980), range(1995, 2050))
        self.assertNoHolidayName("Day of Reconciliation", range(1940, 1995))
        self.assertHolidayName(
            "Day of Reconciliation", (f"{year}-12-16" for year in range(1995, 2005))
        )

    def test_human_rights_day(self):
        name = "Human Rights Day"
        self.assertHolidayName(name, (f"{year}-03-21" for year in range(1995, 2050)))
        self.assertNoHoliday(f"{year}-03-21" for year in range(1940, 1995))
        self.assertNoHolidayName(name, range(1940, 1995))

    def test_freedom_day(self):
        name = "Freedom Day"
        self.assertHolidayName(name, (f"{year}-04-27" for year in range(1995, 2050)))
        self.assertNoHoliday(f"{year}-04-27" for year in range(1940, 1995))
        self.assertNoHolidayName(name, range(1940, 1995))

    def test_workers_day(self):
        name = "Workers' Day"
        self.assertHolidayName(name, (f"{year}-05-01" for year in range(1995, 2050)))
        self.assertNoHoliday(f"{year}-05-01" for year in set(range(1940, 1995)).difference({1987}))
        self.assertNoHolidayName(name, set(range(1940, 1995)).difference(set(range(1987, 1990))))
        self.assertHolidayName(name, "1987-05-01", "1988-05-06", "1989-05-05")

    def test_youth_day(self):
        name = "Youth Day"
        self.assertHolidayName(name, (f"{year}-06-16" for year in range(1995, 2050)))
        self.assertNoHoliday(f"{year}-06-16" for year in range(1940, 1995))
        self.assertNoHolidayName(name, range(1940, 1995))

    def test_national_womens_day(self):
        name = "National Women's Day"
        self.assertHolidayName(name, (f"{year}-08-09" for year in range(1995, 2050)))
        self.assertNoHoliday(f"{year}-08-09" for year in range(1940, 1995))
        self.assertNoHolidayName(name, range(1940, 1995))

    def test_heritage_day(self):
        name = "Heritage Day"
        self.assertHolidayName(name, (f"{year}-09-24" for year in range(1995, 2050)))
        self.assertNoHoliday(f"{year}-09-24" for year in range(1940, 1995))
        self.assertNoHolidayName(name, range(1940, 1995))

    def test_christmas(self):
        self.assertHolidayName("Christmas Day", (f"{year}-12-25" for year in range(1940, 2050)))
        self.assertHolidayName("Day of Goodwill", (f"{year}-12-26" for year in range(1980, 2050)))
        self.assertHolidayName("Boxing Day", (f"{year}-12-26" for year in range(1940, 1980)))
        self.assertNoHolidayName("Day of Goodwill", range(1940, 1980))
        self.assertNoHolidayName("Boxing Day", range(1980, 2050))

    def test_founders_van_riebeecks_day(self):
        name1 = "Van Riebeeck's Day"
        name2 = "Founder's Day"
        self.assertHolidayName(name1, (f"{year}-04-06" for year in range(1952, 1974)))
        self.assertHolidayName(name2, (f"{year}-04-06" for year in range(1980, 1995)))
        self.assertNoHolidayName(name1, range(1940, 1952), range(1975, 2050))
        self.assertNoHolidayName(name2, range(1940, 1980), range(1995, 2050))

    def test_ascension_day(self):
        name = "Ascension Day"
        self.assertHolidayName(
            name,
            "1940-05-02",
            "1950-05-18",
            "1960-05-26",
            "1970-05-07",
            "1980-05-15",
            "1990-05-24",
            "1993-05-20",
        )
        self.assertNoHolidayName(name, range(1994, 2050))

    def test_empire_day(self):
        name = "Empire Day"
        self.assertHolidayName(name, (f"{year}-05-24" for year in range(1940, 1952)))
        self.assertNoHolidayName(name, range(1952, 2050))

    def test_republic_union_day(self):
        name1 = "Union Day"
        name2 = "Republic Day"
        self.assertHolidayName(name1, (f"{year}-05-31" for year in range(1940, 1961)))
        self.assertHolidayName(name2, (f"{year}-05-31" for year in range(1961, 1994)))
        self.assertNoHolidayName(name1, range(1961, 2050))
        self.assertNoHolidayName(name2, range(1940, 1961), range(1994, 2050))

    def test_queens_birthday(self):
        name = "Queen's Birthday"
        self.assertHolidayName(
            name,
            "1952-07-14",
            "1953-07-13",
            "1954-07-12",
            "1955-07-11",
            "1956-07-09",
            "1957-07-08",
            "1958-07-14",
            "1959-07-13",
            "1960-07-11",
        )
        self.assertNoHolidayName(name, range(1940, 1952), range(1961, 2050))

    def test_family_day(self):
        name = "Family Day"
        self.assertHolidayName(name, (f"{year}-07-10" for year in range(1961, 1974)))
        self.assertNoHolidayName(name, range(1940, 1961), range(1974, 1980))

    def test_kings_birthday(self):
        name = "King's Birthday"
        self.assertHolidayName(
            name,
            "1940-08-05",
            "1941-08-04",
            "1942-08-03",
            "1943-08-02",
            "1944-08-07",
            "1945-08-06",
            "1946-08-05",
            "1947-08-04",
            "1948-08-02",
            "1949-08-01",
            "1950-08-07",
            "1951-08-06",
        )
        self.assertNoHolidayName(name, range(1952, 2050))

    def test_settlers_birthday(self):
        name = "Settlers' Day"
        self.assertHolidayName(
            name,
            "1952-09-01",
            "1960-09-05",
            "1970-09-07",
            "1979-09-03",
        )
        self.assertNoHolidayName(name, range(1940, 1952), range(1980, 2050))

    def test_kruger_day(self):
        name = "Kruger Day"
        self.assertHolidayName(name, (f"{year}-10-10" for year in range(1952, 1994)))
        self.assertNoHolidayName(name, range(1940, 1952), range(1994, 2050))

    def test_observed(self):
        dt = (
            # New Year's Day
            "2006-01-02",
            "2012-01-02",
            "2017-01-02",
            "2023-01-02",
            # Human Rights Day
            "2010-03-22",
            "2021-03-22",
            # Freedom Day
            "2008-04-28",
            "2014-04-28",
            # Workers' Day
            "2005-05-02",
            "2011-05-02",
            "2016-05-02",
            "2022-05-02",
            # Youth Day
            "2013-06-17",
            "2019-06-17",
            # National Women's Day
            "2009-08-10",
            "2015-08-10",
            "2020-08-10",
            # Heritage Day
            "2006-09-25",
            "2017-09-25",
            "2023-09-25",
            # Day of Reconciliation
            "2007-12-17",
            "2012-12-17",
            "2018-12-17",
            # Day of Goodwill
            "2010-12-27",
            "2021-12-27",
            # special holiday
            "2000-01-03",
        )
        self.assertHoliday(dt)
        self.assertNoNonObservedHoliday(dt)



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from unittest import TestCase

from holidays.countries.bahamas import Bahamas, BS, BHS
from tests.common import CommonCountryTests


class TestBahamas(CommonCountryTests, TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass(Bahamas, years=range(1974, 2050), years_non_observed=range(1974, 2050))

    def test_country_aliases(self):
        self.assertAliases(Bahamas, BS, BHS)

    def test_no_holidays(self):
        self.assertNoHolidays(Bahamas(years=1973))

    def test_special_public_holidays(self):
        self.assertHoliday("2022-09-19")

    def test_special_observance_2012_and_prior(self):
        self.assertNoNonObservedHoliday(
            # Cases of New Year's Day observance overflow into prev year
            "1979-12-31",
            "1984-12-31",
            "1990-12-31",
            "2001-12-31",
            "2007-12-31",
            # New Year's Day
            "2003-01-03",
            "2004-01-02",
            "2009-01-02",
            "2011-01-03",
            # Discovery Day
            "2000-10-13",
            "2004-10-11",
            "2005-10-14",
            "2006-10-13",
            "2010-10-11",
            "2011-10-14",
        )

    def test_2012(self):
        self.assertHolidays(
            Bahamas(years=2012),
            ("2012-01-01", "New Year's Day"),
            ("2012-01-02", "New Year's Day (observed)"),
            ("2012-04-06", "Good Friday"),
            ("2012-04-09", "Easter Monday"),
            ("2012-05-28", "Whit Monday"),
            ("2012-06-01", "Labour Day"),
            ("2012-07-10", "Independence Day"),
            ("2012-08-06", "Emancipation Day"),
            ("2012-10-12", "Discovery Day"),
            ("2012-12-25", "Christmas Day"),
            ("2012-12-26", "Boxing Day"),
        )

    def test_2013(self):
        self.assertHolidays(
            Bahamas(years=2013),
            ("2013-01-01", "New Year's Day"),
            ("2013-03-29", "Good Friday"),
            ("2013-04-01", "Easter Monday"),
            ("2013-05-20", "Whit Monday"),
            ("2013-06-07", "Randol Fawkes Labour Day"),
            ("2013-07-10", "Independence Day"),
            ("2013-08-05", "Emancipation Day"),
            ("2013-10-14", "National Heroes Day"),
            ("2013-12-25", "Christmas Day"),
            ("2013-12-26", "Boxing Day"),
        )

    def test_2014(self):
        self.assertHolidays(
            Bahamas(years=2014),
            ("2014-01-01", "New Year's Day"),
            ("2014-01-10", "Majority Rule Day"),
            ("2014-04-18", "Good Friday"),
            ("2014-04-21", "Easter Monday"),
            ("2014-06-06", "Randol Fawkes Labour Day"),
            ("2014-06-09", "Whit Monday"),
            ("2014-07-10", "Independence Day"),
            ("2014-08-04", "Emancipation Day"),
            ("2014-10-13", "National Heroes Day"),
            ("2014-12-25", "Christmas Day"),
            ("2014-12-26", "Boxing Day"),
        )

    def test_2020(self):
        self.assertHolidays(
            Bahamas(years=2020),
            ("2020-01-01", "New Year's Day"),
            ("2020-01-10", "Majority Rule Day"),
            ("2020-04-10", "Good Friday"),
            ("2020-04-13", "Easter Monday"),
            ("2020-06-01", "Whit Monday"),
            ("2020-06-05", "Randol Fawkes Labour Day"),
            ("2020-07-10", "Independence Day"),
            ("2020-08-03", "Emancipation Day"),
            ("2020-10-12", "National Heroes Day"),
            ("2020-12-25", "Christmas Day"),
            ("2020-12-26", "Boxing Day"),
            ("2020-12-28", "Boxing Day (observed)"),
        )

    def test_2021(self):
        self.assertHolidays(
            Bahamas(years=2021),
            ("2021-01-01", "New Year's Day"),
            ("2021-01-10", "Majority Rule Day"),
            ("2021-01-11", "Majority Rule Day (observed)"),
            ("2021-04-02", "Good Friday"),
            ("2021-04-05", "Easter Monday"),
            ("2021-05-24", "Whit Monday"),
            ("2021-06-04", "Randol Fawkes Labour Day"),
            ("2021-07-10", "Independence Day"),
            ("2021-07-12", "Independence Day (observed)"),
            ("2021-08-02", "Emancipation Day"),
            ("2021-10-11", "National Heroes Day"),
            ("2021-12-25", "Christmas Day"),
            ("2021-12-26", "Boxing Day"),
            ("2021-12-27", "Boxing Day (observed)"),
        )

    def test_2022(self):
        self.assertHolidays(
            Bahamas(years=2022),
            ("2022-01-01", "New Year's Day"),
            ("2022-01-10", "Majority Rule Day"),
            ("2022-04-15", "Good Friday"),
            ("2022-04-18", "Easter Monday"),
            ("2022-06-03", "Randol Fawkes Labour Day"),
            ("2022-06-06", "Whit Monday"),
            ("2022-07-10", "Independence Day"),
            ("2022-07-11", "Independence Day (observed)"),
            ("2022-08-01", "Emancipation Day"),
            ("2022-09-19", "State Funeral of Queen Elizabeth II"),
            ("2022-10-10", "National Heroes Day"),
            ("2022-12-25", "Christmas Day"),
            ("2022-12-26", "Boxing Day"),
            ("2022-12-27", "Christmas Day (observed)"),
        )

    def test_2023(self):
        self.assertHolidays(
            Bahamas(years=2023),
            ("2023-01-01", "New Year's Day"),
            ("2023-01-02", "New Year's Day (observed)"),
            ("2023-01-10", "Majority Rule Day"),
            ("2023-04-07", "Good Friday"),
            ("2023-04-10", "Easter Monday"),
            ("2023-05-29", "Whit Monday"),
            ("2023-06-02", "Randol Fawkes Labour Day"),
            ("2023-07-10", "Independence Day"),
            ("2023-08-07", "Emancipation Day"),
            ("2023-10-09", "National Heroes Day"),
            ("2023-12-25", "Christmas Day"),
            ("2023-12-26", "Boxing Day"),
        )



#  holidays
#  --------
#  A fast, efficient Python library for generating country, province and state
#  specific sets of holidays on the fly. It aims to make determining whether a
#  specific date is a holiday as fast and flexible as possible.
#
#  Authors: Vacanza Team and individual contributors (see AUTHORS file)
#           dr-prodigy <dr.prodigy.github@gmail.com> (c) 2017-2023
#           ryanss <ryanssdev@icloud.com> (c) 2014-2017
#  Website: https://github.com/vacanza/holidays
#  License: MIT (see LICENSE file)

from datetime import date
from typing import Optional

from holidays.calendars.gregorian import JAN, APR, MAY, JUN, JUL, SEP, OCT, DEC
from holidays.groups import ChristianHolidays, InternationalHolidays, StaticHolidays
from holidays.observed_holiday_base import (
    ObservedHolidayBase,
    SAT_SUN_TO_NEXT_WORKDAY,
    SUN_TO_NEXT_WORKDAY,
)


class Guernsey(ObservedHolidayBase, ChristianHolidays, InternationalHolidays, StaticHolidays):
    """
    References:
      - https://en.wikipedia.org/wiki/Public_holidays_in_Guernsey
      - https://guernseylegalresources.gg/ordinances/guernsey-bailiwick/p/public-holidays/
      - https://www.thegazette.co.uk/all-notices
    Checked with:
      - https://www.gov.gg/holidaydates

    His/Her Majesty's Birthday pre-1946 is cross-checked with The London Gazette's Record,
    Specifically as "Home Station" entry under King's Birthday declaration lists.

    Since 1955, if a bank holiday is on a sunday, a substitute weekday becomes a bank holiday,
    normally the following Monday. From 2009 onwards this also applies to saturday as well.
    """

    country = "GG"
    observed_label = "%s (substitute day)"
    # Ordonnance relative aux Jours Fériés, 1909.
    start_year = 1909

    def __init__(self, *args, **kwargs):
        ChristianHolidays.__init__(self)
        InternationalHolidays.__init__(self)
        StaticHolidays.__init__(self, GuernseyStaticHolidays)
        kwargs.setdefault("observed_rule", SAT_SUN_TO_NEXT_WORKDAY)
        kwargs.setdefault("observed_since", 1955)
        ObservedHolidayBase.__init__(self, *args, **kwargs)

    def _add_observed(self, dt: date, **kwargs) -> tuple[bool, Optional[date]]:
        # Prior to 2009, in-lieu are only given for Sundays.
        # https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55179
        kwargs.setdefault(
            "rule", SUN_TO_NEXT_WORKDAY if dt < date(2009, DEC, 15) else self._observed_rule
        )
        return super()._add_observed(dt, **kwargs)

    def _populate_public_holidays(self) -> None:
        # New Year's Day
        # Le jour de l'An

        # New Year's Day.
        self._add_observed(self._add_new_years_day("New Year's Day"))

        # Good Friday
        # Vendredi Saint

        # Good Friday.
        self._add_good_friday("Good Friday")

        # Easter Monday
        # Le Lundi de Pâques

        # Easter Monday.
        self._add_easter_monday("Easter Monday")

        # May Day
        # (No official Guernésiais translation available)
        # Started in 1980 for Guernsey.
        # Moved to May 8 specifically for 1995 and 2020.

        if self._year >= 1980:
            # May Day Bank Holiday
            name = "May Day Bank Holiday"
            if self._year in {1995, 2020}:
                self._add_holiday_may_8(name)
            else:
                self._add_holiday_1st_mon_of_may(name)

        # Spring bank holiday
        # (No official Guernésiais translation available)
        # Replaced Whit Monday in 1967, 1970, 1973, 1974, 1975, 1976.
        # No observance in 1977-1978. Properly established from 1979 onwards.
        # Moved for Queen Elizabeth II's Jubilee Dates in 2002, 2012, and 2022.

        if self._year in {1967, 1970} or 1973 <= self._year <= 1976 or self._year >= 1979:
            spring_bank_dates = {
                2002: (JUN, 4),
                2012: (JUN, 4),
                2022: (JUN, 2),
            }
            # Spring Bank Holiday
            name = "Spring Bank Holiday"
            if self._year in spring_bank_dates:
                self._add_holiday(name, spring_bank_dates[self._year])
            else:
                self._add_holiday_last_mon_of_may(name)

        # Whit Monday
        # Le lundi de Pentecôte
        # Replaced by Spring Bank Holiday in 1967, 1970, 1973, 1974, 1975, 1976.
        # Fully Removed in 1977.

        if self._year <= 1966 or self._year in {1968, 1969, 1971, 1972}:
            # Whit Monday.
            self._add_whit_monday("Whit Monday")

        # His/Her Majesty's Birthday
        # La jour fixé pour la célébration du jour de naissanee de Sa Majesté
        # Special Non-Observance Decree was issued in 1940.
        # This was fully replaced by Liberation Day in 1947.

        if self._year <= 1946:
            his_majesty_birth_dates = {
                # Edward VII (MAY/JUN observance).
                1909: (JUN, 25),  # (NOV, 9) for overseas.
                # George V.
                1910: (JUN, 24),  # (NOV, 9) for overseas; No decree cancelled Edward VII ones.
                1911: (MAY, 27),  # (JUN, 3) for overseas.
                1912: (JUN, 14),  # (JUN, 3) for overseas.
                1913: (JUN, 3),
                1914: (JUN, 24),  # Specifically held on the 24th on Guernsey, is 22nd elsewhere.
                # Not observed in 1915-1918 due to WW1.
                1919: (JUN, 3),
                1920: (JUN, 5),  # (JUN, 3) for overseas.
                1921: (JUN, 4),  # (JUN, 3) for overseas.
                1922: (JUN, 3),
                1923: (JUN, 2),
                1924: (JUN, 3),
                1925: (JUN, 3),
                1926: (JUN, 5),
                1927: (JUN, 3),
                1928: (JUN, 4),
                1929: (JUN, 3),
                1930: (JUN, 3),
                1931: (JUN, 3),
                1932: (JUN, 3),
                1933: (JUN, 3),
                1934: (JUN, 4),
                1935: (JUN, 3),
                # Edward VIII.
                1936: (JUN, 23),
                # George VI.
                1937: (JUN, 9),
                1938: (JUN, 9),
                1939: (JUN, 8),
                # Not observed in 1940-1941 due to WW2.
                # Although this was only de jure cancelled in 1940 for Guernsey.
                1942: (JUN, 11),
                1943: (JUN, 2),
                1944: (JUN, 8),
                1945: (JUN, 14),
                1946: (JUN, 13),
            }
            if self._year in his_majesty_birth_dates:
                # His Majesty's Birthday.
                self._add_holiday("His Majesty's Birthday", his_majesty_birth_dates[self._year])

        # Summer Bank Holiday
        # Le premier Lundi du mois d'Août
        # Current Pattern started in 1965. Was initially on first Monday of August for Guernsey.
        # Briefly as first Monday of September between 1968-1969 and no observance for 1977-1978.

        # Summer Bank Holiday.
        summer_bank_holiday = "Summer Bank Holiday"
        if self._year <= 1976 or self._year >= 1979:
            if 1968 <= self._year <= 1969:
                self._add_holiday_1st_mon_of_sep(summer_bank_holiday)
            elif self._year >= 1965:
                self._add_holiday_last_mon_of_aug(summer_bank_holiday)
            else:
                self._add_holiday_1st_mon_of_aug(summer_bank_holiday)

        # Christmas Day
        # Le jour de Noël

        # Christmas Day
        christmas_day = self._add_christmas_day("Christmas Day")

        # Boxing Day
        # Le premier jour ouvrier après le jour de Noël

        # Boxing Day
        boxing_day = self._add_christmas_day_two("Boxing Day")

        self._add_observed(christmas_day)
        self._add_observed(boxing_day)

        # Guernsey exclusive holidays

        # Liberation Day
        # Le jour fixé par les Etats pour la célébration de la Libération de cette Ile
        # de l'occupation allemande par les Forces Britannique
        # Started in 1946 for Guernsey. Fully replaced HM Birthdays in 1947. This has no in-lieus.
        # Specifically moved to May 10 in 2010.
        # Unlike Jersey this wasn't moved in 2021.

        if self._year >= 1946:
            liberation_dates = {
                2010: (MAY, 10),
            }
            # Liberation Day.
            name = "Liberation Day"
            if self._year in liberation_dates:
                self._add_holiday(name, liberation_dates[self._year])
            else:
                self._add_holiday_may_9(name)


class GG(Guernsey):
    pass


class GGY(Guernsey):
    pass


class GuernseyStaticHolidays:
    """
    References:
      - https://www.gov.gg/holidaydates
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=52182
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=52183
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=52184
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=52185
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=52188
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=52189
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=52631
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55175
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55176
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55177
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55179
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55180
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55183
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55185
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=55505
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=57425
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=60605
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=60606
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=60607
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=60608
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=60613
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=60614
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=79939
      - https://guernseylegalresources.gg/CHttpHandler.ashx?documentid=83147
      - https://www.bbc.com/news/world-europe-guernsey-62864318
      - https://www.bbc.co.uk/news/articles/c1441ddn87po

    All "jour de relâche" entries are translated as special day off instead.

    While there's no source for Queen Elizabeth II's Silver Jubilee being observed
    by Guernsey - it's safe to assume such holiday was declared.
    """

    # Special Day Off.
    special_day_off = "Special Day Off"

    # The visit of Her Majesty Queen Elizabeth II.
    elizabeth_2_royal_visit = "The visit of Her Majesty Queen Elizabeth II"

    # Millenium Celebrations.
    millenium_celebrations = "Millennium Celebrations"

    # New Year's Day.
    new_years_day_in_lieu = "New Year's Day"

    # Christmas Day.
    christmas_day_in_lieu = "Christmas Day"

    # Boxing Day.
    boxing_day_in_lieu = "Boxing Day"

    special_public_holidays = {
        1930: (OCT, 10, special_day_off),
        1935: (MAY, 6, "Silver Jubilee of King George V"),
        1937: (MAY, 12, "The Coronation of King George VI and Queen Elizabeth"),
        1939: (SEP, 4, special_day_off),
        1949: (SEP, 19, special_day_off),
        1953: (JUN, 2, "The Coronation of Queen Elizabeth II"),
        1957: (JUL, 26, elizabeth_2_royal_visit),
        1977: (JUN, 7, "Silver Jubilee of Elizabeth II"),
        1978: (JUN, 28, elizabeth_2_royal_visit),
        1981: (JUL, 29, "Wedding of Charles and Diana"),
        1989: (MAY, 23, elizabeth_2_royal_visit),
        1999: (
            (DEC, 28, millenium_celebrations),
            (DEC, 31, millenium_celebrations),
        ),
        2000: (JAN, 3, millenium_celebrations),
        2001: (JUL, 12, elizabeth_2_royal_visit),
        2002: (JUN, 3, "Golden Jubilee of Elizabeth II"),
        2011: (APR, 29, "Wedding of William and Catherine"),
        2012: (JUN, 5, "Diamond Jubilee of Elizabeth II"),
        2022: (
            (JUN, 3, "Queen's Platinum Jubilee Bank Holiday"),
            (SEP, 19, "State Funeral of Queen Elizabeth II"),
        ),
        2023: (MAY, 8, "Extra Public Holiday for the Coronation of King Charles III"),
        2024: (JUL, 16, "The visit of His Majesty King Charles III and Queen Camilla"),
    }
    special_public_holidays_observed = {
        1932: (DEC, 27, christmas_day_in_lieu),
        1933: (JAN, 2, new_years_day_in_lieu),
        1938: (DEC, 27, christmas_day_in_lieu),
        1939: (JAN, 2, new_years_day_in_lieu),
        1981: (DEC, 28, boxing_day_in_lieu),
        1982: (DEC, 28, christmas_day_in_lieu),
        1983: (JAN, 3, new_years_day_in_lieu),
        1987: (DEC, 28, boxing_day_in_lieu),
        1993: (DEC, 28, christmas_day_in_lieu),
        1994: (JAN, 3, new_years_day_in_lieu),
        2004: (DEC, 28, christmas_day_in_lieu),
        2005: (JAN, 3, new_years_day_in_lieu),
    }
